"""Generated message classes for batch version v1.

An API to manage the running of Batch resources on Google Cloud Platform.
"""
# NOTE: This file is autogenerated and should not be edited by hand.

from apitools.base.protorpclite import messages as _messages
from apitools.base.py import encoding
from apitools.base.py import extra_types


package = 'batch'


class Accelerator(_messages.Message):
  r"""Accelerator describes Compute Engine accelerators to be attached to the
  VM.

  Fields:
    count: The number of accelerators of this type.
    driverVersion: Optional. The NVIDIA GPU driver version that should be
      installed for this type. You can define the specific driver version such
      as "470.103.01", following the driver version requirements in
      https://cloud.google.com/compute/docs/gpus/install-drivers-gpu#minimum-
      driver. Batch will install the specific accelerator driver if qualified.
    installGpuDrivers: Deprecated: please use instances[0].install_gpu_drivers
      instead.
    type: The accelerator type. For example, "nvidia-tesla-t4". See `gcloud
      compute accelerator-types list`.
  """

  count = _messages.IntegerField(1)
  driverVersion = _messages.StringField(2)
  installGpuDrivers = _messages.BooleanField(3)
  type = _messages.StringField(4)


class ActionCondition(_messages.Message):
  r"""Conditions for actions to deal with task failures.

  Fields:
    exitCodes: Exit codes of a task execution. If there are more than 1 exit
      codes, when task executes with any of the exit code in the list, the
      condition is met and the action will be executed.
  """

  exitCodes = _messages.IntegerField(1, repeated=True, variant=_messages.Variant.INT32)


class AllocationPolicy(_messages.Message):
  r"""A Job's resource allocation policy describes when, where, and how
  compute resources should be allocated for the Job.

  Messages:
    LabelsValue: Custom labels to apply to the job and all the Compute Engine
      resources that both are created by this allocation policy and support
      labels. Use labels to group and describe the resources they are applied
      to. Batch automatically applies predefined labels and supports multiple
      `labels` fields for each job, which each let you apply custom labels to
      various resources. Label names that start with "goog-" or "google-" are
      reserved for predefined labels. For more information about labels with
      Batch, see [Organize resources using
      labels](https://cloud.google.com/batch/docs/organize-resources-using-
      labels).

  Fields:
    instances: Describe instances that can be created by this
      AllocationPolicy. Only instances[0] is supported now.
    labels: Custom labels to apply to the job and all the Compute Engine
      resources that both are created by this allocation policy and support
      labels. Use labels to group and describe the resources they are applied
      to. Batch automatically applies predefined labels and supports multiple
      `labels` fields for each job, which each let you apply custom labels to
      various resources. Label names that start with "goog-" or "google-" are
      reserved for predefined labels. For more information about labels with
      Batch, see [Organize resources using
      labels](https://cloud.google.com/batch/docs/organize-resources-using-
      labels).
    location: Location where compute resources should be allocated for the
      Job.
    network: The network policy. If you define an instance template in the
      `InstancePolicyOrTemplate` field, Batch will use the network settings in
      the instance template instead of this field.
    placement: The placement policy.
    serviceAccount: Defines the service account for Batch-created VMs. If
      omitted, the [default Compute Engine service
      account](https://cloud.google.com/compute/docs/access/service-
      accounts#default_service_account) is used. Must match the service
      account specified in any used instance template configured in the Batch
      job. Includes the following fields: * email: The service account's email
      address. If not set, the default Compute Engine service account is used.
      * scopes: Additional OAuth scopes to grant the service account, beyond
      the default cloud-platform scope. (list of strings)
    tags: Optional. Tags applied to the VM instances. The tags identify valid
      sources or targets for network firewalls. Each tag must be 1-63
      characters long, and comply with
      [RFC1035](https://www.ietf.org/rfc/rfc1035.txt).
  """

  @encoding.MapUnrecognizedFields('additionalProperties')
  class LabelsValue(_messages.Message):
    r"""Custom labels to apply to the job and all the Compute Engine resources
    that both are created by this allocation policy and support labels. Use
    labels to group and describe the resources they are applied to. Batch
    automatically applies predefined labels and supports multiple `labels`
    fields for each job, which each let you apply custom labels to various
    resources. Label names that start with "goog-" or "google-" are reserved
    for predefined labels. For more information about labels with Batch, see
    [Organize resources using
    labels](https://cloud.google.com/batch/docs/organize-resources-using-
    labels).

    Messages:
      AdditionalProperty: An additional property for a LabelsValue object.

    Fields:
      additionalProperties: Additional properties of type LabelsValue
    """

    class AdditionalProperty(_messages.Message):
      r"""An additional property for a LabelsValue object.

      Fields:
        key: Name of the additional property.
        value: A string attribute.
      """

      key = _messages.StringField(1)
      value = _messages.StringField(2)

    additionalProperties = _messages.MessageField('AdditionalProperty', 1, repeated=True)

  instances = _messages.MessageField('InstancePolicyOrTemplate', 1, repeated=True)
  labels = _messages.MessageField('LabelsValue', 2)
  location = _messages.MessageField('LocationPolicy', 3)
  network = _messages.MessageField('NetworkPolicy', 4)
  placement = _messages.MessageField('PlacementPolicy', 5)
  serviceAccount = _messages.MessageField('ServiceAccount', 6)
  tags = _messages.StringField(7, repeated=True)


class AttachedDisk(_messages.Message):
  r"""A new or an existing persistent disk (PD) or a local ssd attached to a
  VM instance.

  Fields:
    deviceName: Device name that the guest operating system will see. It is
      used by Runnable.volumes field to mount disks. So please specify the
      device_name if you want Batch to help mount the disk, and it should
      match the device_name field in volumes.
    existingDisk: Name of an existing PD.
    newDisk: A Disk attribute.
  """

  deviceName = _messages.StringField(1)
  existingDisk = _messages.StringField(2)
  newDisk = _messages.MessageField('Disk', 3)


class Barrier(_messages.Message):
  r"""A barrier runnable automatically blocks the execution of subsequent
  runnables until all the tasks in the task group reach the barrier.

  Fields:
    name: Barriers are identified by their index in runnable list. Names are
      not required, but if present should be an identifier.
  """

  name = _messages.StringField(1)


class BatchProjectsLocationsGetRequest(_messages.Message):
  r"""A BatchProjectsLocationsGetRequest object.

  Fields:
    name: Resource name for the location.
  """

  name = _messages.StringField(1, required=True)


class BatchProjectsLocationsJobsCancelRequest(_messages.Message):
  r"""A BatchProjectsLocationsJobsCancelRequest object.

  Fields:
    cancelJobRequest: A CancelJobRequest resource to be passed as the request
      body.
    name: Required. Job name.
  """

  cancelJobRequest = _messages.MessageField('CancelJobRequest', 1)
  name = _messages.StringField(2, required=True)


class BatchProjectsLocationsJobsCreateRequest(_messages.Message):
  r"""A BatchProjectsLocationsJobsCreateRequest object.

  Fields:
    job: A Job resource to be passed as the request body.
    jobId: ID used to uniquely identify the Job within its parent scope. This
      field should contain at most 63 characters and must start with lowercase
      characters. Only lowercase characters, numbers and '-' are accepted. The
      '-' character cannot be the first or the last one. A system generated ID
      will be used if the field is not set. The job.name field in the request
      will be ignored and the created resource name of the Job will be
      "{parent}/jobs/{job_id}".
    parent: Required. The parent resource name where the Job will be created.
      Pattern: "projects/{project}/locations/{location}"
    requestId: Optional. An optional request ID to identify requests. Specify
      a unique request ID so that if you must retry your request, the server
      will know to ignore the request if it has already been completed. The
      server will guarantee that for at least 60 minutes since the first
      request. For example, consider a situation where you make an initial
      request and the request times out. If you make the request again with
      the same request ID, the server can check if original operation with the
      same request ID was received, and if so, will ignore the second request.
      This prevents clients from accidentally creating duplicate commitments.
      The request ID must be a valid UUID with the exception that zero UUID is
      not supported (00000000-0000-0000-0000-000000000000).
  """

  job = _messages.MessageField('Job', 1)
  jobId = _messages.StringField(2)
  parent = _messages.StringField(3, required=True)
  requestId = _messages.StringField(4)


class BatchProjectsLocationsJobsDeleteRequest(_messages.Message):
  r"""A BatchProjectsLocationsJobsDeleteRequest object.

  Fields:
    name: Job name.
    reason: Optional. Reason for this deletion.
    requestId: Optional. An optional request ID to identify requests. Specify
      a unique request ID so that if you must retry your request, the server
      will know to ignore the request if it has already been completed. The
      server will guarantee that for at least 60 minutes after the first
      request. For example, consider a situation where you make an initial
      request and the request times out. If you make the request again with
      the same request ID, the server can check if original operation with the
      same request ID was received, and if so, will ignore the second request.
      This prevents clients from accidentally creating duplicate commitments.
      The request ID must be a valid UUID with the exception that zero UUID is
      not supported (00000000-0000-0000-0000-000000000000).
  """

  name = _messages.StringField(1, required=True)
  reason = _messages.StringField(2)
  requestId = _messages.StringField(3)


class BatchProjectsLocationsJobsGetRequest(_messages.Message):
  r"""A BatchProjectsLocationsJobsGetRequest object.

  Fields:
    name: Required. Job name.
  """

  name = _messages.StringField(1, required=True)


class BatchProjectsLocationsJobsListRequest(_messages.Message):
  r"""A BatchProjectsLocationsJobsListRequest object.

  Fields:
    filter: List filter.
    orderBy: Optional. Sort results. Supported are "name", "name desc",
      "create_time", and "create_time desc".
    pageSize: Page size.
    pageToken: Page token.
    parent: Parent path.
  """

  filter = _messages.StringField(1)
  orderBy = _messages.StringField(2)
  pageSize = _messages.IntegerField(3, variant=_messages.Variant.INT32)
  pageToken = _messages.StringField(4)
  parent = _messages.StringField(5, required=True)


class BatchProjectsLocationsJobsTaskGroupsTasksGetRequest(_messages.Message):
  r"""A BatchProjectsLocationsJobsTaskGroupsTasksGetRequest object.

  Fields:
    name: Required. Task name.
  """

  name = _messages.StringField(1, required=True)


class BatchProjectsLocationsJobsTaskGroupsTasksListRequest(_messages.Message):
  r"""A BatchProjectsLocationsJobsTaskGroupsTasksListRequest object.

  Fields:
    filter: Task filter, null filter matches all Tasks. Filter string should
      be of the format State=TaskStatus.State e.g. State=RUNNING
    pageSize: Page size.
    pageToken: Page token.
    parent: Required. Name of a TaskGroup from which Tasks are being
      requested. Pattern: "projects/{project}/locations/{location}/jobs/{job}/
      taskGroups/{task_group}"
  """

  filter = _messages.StringField(1)
  pageSize = _messages.IntegerField(2, variant=_messages.Variant.INT32)
  pageToken = _messages.StringField(3)
  parent = _messages.StringField(4, required=True)


class BatchProjectsLocationsListRequest(_messages.Message):
  r"""A BatchProjectsLocationsListRequest object.

  Fields:
    extraLocationTypes: Optional. Do not use this field unless explicitly
      documented otherwise. This is primarily for internal usage.
    filter: A filter to narrow down results to a preferred subset. The
      filtering language accepts strings like `"displayName=tokyo"`, and is
      documented in more detail in [AIP-160](https://google.aip.dev/160).
    name: The resource that owns the locations collection, if applicable.
    pageSize: The maximum number of results to return. If not set, the service
      selects a default.
    pageToken: A page token received from the `next_page_token` field in the
      response. Send that page token to receive the subsequent page.
  """

  extraLocationTypes = _messages.StringField(1, repeated=True)
  filter = _messages.StringField(2)
  name = _messages.StringField(3, required=True)
  pageSize = _messages.IntegerField(4, variant=_messages.Variant.INT32)
  pageToken = _messages.StringField(5)


class BatchProjectsLocationsOperationsCancelRequest(_messages.Message):
  r"""A BatchProjectsLocationsOperationsCancelRequest object.

  Fields:
    cancelOperationRequest: A CancelOperationRequest resource to be passed as
      the request body.
    name: The name of the operation resource to be cancelled.
  """

  cancelOperationRequest = _messages.MessageField('CancelOperationRequest', 1)
  name = _messages.StringField(2, required=True)


class BatchProjectsLocationsOperationsDeleteRequest(_messages.Message):
  r"""A BatchProjectsLocationsOperationsDeleteRequest object.

  Fields:
    name: The name of the operation resource to be deleted.
  """

  name = _messages.StringField(1, required=True)


class BatchProjectsLocationsOperationsGetRequest(_messages.Message):
  r"""A BatchProjectsLocationsOperationsGetRequest object.

  Fields:
    name: The name of the operation resource.
  """

  name = _messages.StringField(1, required=True)


class BatchProjectsLocationsOperationsListRequest(_messages.Message):
  r"""A BatchProjectsLocationsOperationsListRequest object.

  Fields:
    filter: The standard list filter.
    name: The name of the operation's parent resource.
    pageSize: The standard list page size.
    pageToken: The standard list page token.
    returnPartialSuccess: When set to `true`, operations that are reachable
      are returned as normal, and those that are unreachable are returned in
      the ListOperationsResponse.unreachable field. This can only be `true`
      when reading across collections. For example, when `parent` is set to
      `"projects/example/locations/-"`. This field is not supported by default
      and will result in an `UNIMPLEMENTED` error if set unless explicitly
      documented otherwise in service or product specific documentation.
  """

  filter = _messages.StringField(1)
  name = _messages.StringField(2, required=True)
  pageSize = _messages.IntegerField(3, variant=_messages.Variant.INT32)
  pageToken = _messages.StringField(4)
  returnPartialSuccess = _messages.BooleanField(5)


class CancelJobRequest(_messages.Message):
  r"""CancelJob Request.

  Fields:
    requestId: Optional. An optional request ID to identify requests. Specify
      a unique request ID so that if you must retry your request, the server
      will know to ignore the request if it has already been completed. The
      server will guarantee that for at least 60 minutes after the first
      request. For example, consider a situation where you make an initial
      request and the request times out. If you make the request again with
      the same request ID, the server can check if original operation with the
      same request ID was received, and if so, will ignore the second request.
      This prevents clients from accidentally creating duplicate commitments.
      The request ID must be a valid UUID with the exception that zero UUID is
      not supported (00000000-0000-0000-0000-000000000000).
  """

  requestId = _messages.StringField(1)


class CancelOperationRequest(_messages.Message):
  r"""The request message for Operations.CancelOperation."""


class CloudLoggingOption(_messages.Message):
  r"""`CloudLoggingOption` contains additional settings for Cloud Logging logs
  generated by Batch job.

  Fields:
    useGenericTaskMonitoredResource: Optional. Set this field to `true` to
      change the [monitored resource
      type](https://cloud.google.com/monitoring/api/resources) for Cloud
      Logging logs generated by this Batch job from the [`batch.googleapis.com
      /Job`](https://cloud.google.com/monitoring/api/resources#tag_batch.googl
      eapis.com/Job) type to the formerly used [`generic_task`](https://cloud.
      google.com/monitoring/api/resources#tag_generic_task) type.
  """

  useGenericTaskMonitoredResource = _messages.BooleanField(1)


class ComputeResource(_messages.Message):
  r"""Compute resource requirements. ComputeResource defines the amount of
  resources required for each task. Make sure your tasks have enough resources
  to successfully run. If you also define the types of resources for a job to
  use with the [InstancePolicyOrTemplate](https://cloud.google.com/batch/docs/
  reference/rest/v1/projects.locations.jobs#instancepolicyortemplate) field,
  make sure both fields are compatible with each other.

  Fields:
    bootDiskMib: Extra boot disk size in MiB for each task.
    cpuMilli: The milliCPU count. `cpuMilli` defines the amount of CPU
      resources per task in milliCPU units. For example, `1000` corresponds to
      1 vCPU per task. If undefined, the default value is `2000`. If you also
      define the VM's machine type using the `machineType` in [InstancePolicy]
      (https://cloud.google.com/batch/docs/reference/rest/v1/projects.location
      s.jobs#instancepolicy) field or inside the `instanceTemplate` in the [In
      stancePolicyOrTemplate](https://cloud.google.com/batch/docs/reference/re
      st/v1/projects.locations.jobs#instancepolicyortemplate) field, make sure
      the CPU resources for both fields are compatible with each other and
      with how many tasks you want to allow to run on the same VM at the same
      time. For example, if you specify the `n2-standard-2` machine type,
      which has 2 vCPUs each, you are recommended to set `cpuMilli` no more
      than `2000`, or you are recommended to run two tasks on the same VM if
      you set `cpuMilli` to `1000` or less.
    memoryMib: Memory in MiB. `memoryMib` defines the amount of memory per
      task in MiB units. If undefined, the default value is `2000`. If you
      also define the VM's machine type using the `machineType` in [InstancePo
      licy](https://cloud.google.com/batch/docs/reference/rest/v1/projects.loc
      ations.jobs#instancepolicy) field or inside the `instanceTemplate` in
      the [InstancePolicyOrTemplate](https://cloud.google.com/batch/docs/refer
      ence/rest/v1/projects.locations.jobs#instancepolicyortemplate) field,
      make sure the memory resources for both fields are compatible with each
      other and with how many tasks you want to allow to run on the same VM at
      the same time. For example, if you specify the `n2-standard-2` machine
      type, which has 8 GiB each, you are recommended to set `memoryMib` to no
      more than `8192`, or you are recommended to run two tasks on the same VM
      if you set `memoryMib` to `4096` or less.
  """

  bootDiskMib = _messages.IntegerField(1)
  cpuMilli = _messages.IntegerField(2)
  memoryMib = _messages.IntegerField(3)


class Container(_messages.Message):
  r"""Container runnable.

  Fields:
    blockExternalNetwork: If set to true, external network access to and from
      container will be blocked, containers that are with
      block_external_network as true can still communicate with each other,
      network cannot be specified in the `container.options` field.
    commands: Required for some container images. Overrides the `CMD`
      specified in the container. If there is an `ENTRYPOINT` (either in the
      container image or with the `entrypoint` field below) then these
      commands are appended as arguments to the `ENTRYPOINT`.
    enableImageStreaming: Optional. If set to true, this container runnable
      uses Image streaming. Use Image streaming to allow the runnable to
      initialize without waiting for the entire container image to download,
      which can significantly reduce startup time for large container images.
      When `enableImageStreaming` is set to true, the container runtime is
      [containerd](https://containerd.io/) instead of Docker. Additionally,
      this container runnable only supports the following `container`
      subfields: `imageUri`, `commands[]`, `entrypoint`, and `volumes[]`; any
      other `container` subfields are ignored. For more information about the
      requirements and limitations for using Image streaming with Batch, see
      the [`image-streaming` sample on
      GitHub](https://github.com/GoogleCloudPlatform/batch-
      samples/tree/main/api-samples/image-streaming).
    entrypoint: Required for some container images. Overrides the `ENTRYPOINT`
      specified in the container.
    imageUri: Required. The URI to pull the container image from.
    options: Required for some container images. Arbitrary additional options
      to include in the `docker run` command when running this container-for
      example, `--network host`. For the `--volume` option, use the `volumes`
      field for the container.
    password: Required if the container image is from a private Docker
      registry. The password to login to the Docker registry that contains the
      image. For security, it is strongly recommended to specify an encrypted
      password by using a Secret Manager secret:
      `projects/*/secrets/*/versions/*`. Warning: If you specify the password
      using plain text, you risk the password being exposed to any users who
      can view the job or its logs. To avoid this risk, specify a secret that
      contains the password instead. Learn more about [Secret
      Manager](https://cloud.google.com/secret-manager/docs/) and [using
      Secret Manager with Batch](https://cloud.google.com/batch/docs/create-
      run-job-secret-manager).
    username: Required if the container image is from a private Docker
      registry. The username to login to the Docker registry that contains the
      image. You can either specify the username directly by using plain text
      or specify an encrypted username by using a Secret Manager secret:
      `projects/*/secrets/*/versions/*`. However, using a secret is
      recommended for enhanced security. Caution: If you specify the username
      using plain text, you risk the username being exposed to any users who
      can view the job or its logs. To avoid this risk, specify a secret that
      contains the username instead. Learn more about [Secret
      Manager](https://cloud.google.com/secret-manager/docs/) and [using
      Secret Manager with Batch](https://cloud.google.com/batch/docs/create-
      run-job-secret-manager).
    volumes: Volumes to mount (bind mount) from the host machine files or
      directories into the container, formatted to match `--volume` option for
      the `docker run` command-for example, `/foo:/bar` or `/foo:/bar:ro`. If
      the `TaskSpec.Volumes` field is specified but this field is not, Batch
      will mount each volume from the host machine to the container with the
      same mount path by default. In this case, the default mount option for
      containers will be read-only (`ro`) for existing persistent disks and
      read-write (`rw`) for other volume types, regardless of the original
      mount options specified in `TaskSpec.Volumes`. If you need different
      mount settings, you can explicitly configure them in this field.
  """

  blockExternalNetwork = _messages.BooleanField(1)
  commands = _messages.StringField(2, repeated=True)
  enableImageStreaming = _messages.BooleanField(3)
  entrypoint = _messages.StringField(4)
  imageUri = _messages.StringField(5)
  options = _messages.StringField(6)
  password = _messages.StringField(7)
  username = _messages.StringField(8)
  volumes = _messages.StringField(9, repeated=True)


class Disk(_messages.Message):
  r"""A new persistent disk or a local ssd. A VM can only have one local SSD
  setting but multiple local SSD partitions. See
  https://cloud.google.com/compute/docs/disks#pdspecs and
  https://cloud.google.com/compute/docs/disks#localssds.

  Fields:
    diskInterface: Local SSDs are available through both "SCSI" and "NVMe"
      interfaces. If not indicated, "NVMe" will be the default one for local
      ssds. This field is ignored for persistent disks as the interface is
      chosen automatically. See
      https://cloud.google.com/compute/docs/disks/persistent-
      disks#choose_an_interface.
    image: URL for a VM image to use as the data source for this disk. For
      example, the following are all valid URLs: * Specify the image by its
      family name: projects/{project}/global/images/family/{image_family} *
      Specify the image version:
      projects/{project}/global/images/{image_version} You can also use Batch
      customized image in short names. The following image values are
      supported for a boot disk: * `batch-debian`: use Batch Debian images. *
      `batch-cos`: use Batch Container-Optimized images. * `batch-hpc-rocky`:
      use Batch HPC Rocky Linux images.
    sizeGb: Disk size in GB. **Non-Boot Disk**: If the `type` specifies a
      persistent disk, this field is ignored if `data_source` is set as
      `image` or `snapshot`. If the `type` specifies a local SSD, this field
      should be a multiple of 375 GB, otherwise, the final size will be the
      next greater multiple of 375 GB. **Boot Disk**: Batch will calculate the
      boot disk size based on source image and task requirements if you do not
      speicify the size. If both this field and the `boot_disk_mib` field in
      task spec's `compute_resource` are defined, Batch will only honor this
      field. Also, this field should be no smaller than the source disk's size
      when the `data_source` is set as `snapshot` or `image`. For example, if
      you set an image as the `data_source` field and the image's default disk
      size 30 GB, you can only use this field to make the disk larger or equal
      to 30 GB.
    snapshot: Name of a snapshot used as the data source. Snapshot is not
      supported as boot disk now.
    type: Disk type as shown in `gcloud compute disk-types list`. For example,
      local SSD uses type "local-ssd". Persistent disks and boot disks use
      "pd-balanced", "pd-extreme", "pd-ssd" or "pd-standard". If not
      specified, "pd-standard" will be used as the default type for non-boot
      disks, "pd-balanced" will be used as the default type for boot disks.
  """

  diskInterface = _messages.StringField(1)
  image = _messages.StringField(2)
  sizeGb = _messages.IntegerField(3)
  snapshot = _messages.StringField(4)
  type = _messages.StringField(5)


class Empty(_messages.Message):
  r"""A generic empty message that you can re-use to avoid defining duplicated
  empty messages in your APIs. A typical example is to use it as the request
  or the response type of an API method. For instance: service Foo { rpc
  Bar(google.protobuf.Empty) returns (google.protobuf.Empty); }
  """



class Environment(_messages.Message):
  r"""An Environment describes a collection of environment variables to set
  when executing Tasks.

  Messages:
    SecretVariablesValue: A map of environment variable names to Secret
      Manager secret names. The VM will access the named secrets to set the
      value of each environment variable.
    VariablesValue: A map of environment variable names to values.

  Fields:
    encryptedVariables: An encrypted JSON dictionary where the key/value pairs
      correspond to environment variable names and their values.
    secretVariables: A map of environment variable names to Secret Manager
      secret names. The VM will access the named secrets to set the value of
      each environment variable.
    variables: A map of environment variable names to values.
  """

  @encoding.MapUnrecognizedFields('additionalProperties')
  class SecretVariablesValue(_messages.Message):
    r"""A map of environment variable names to Secret Manager secret names.
    The VM will access the named secrets to set the value of each environment
    variable.

    Messages:
      AdditionalProperty: An additional property for a SecretVariablesValue
        object.

    Fields:
      additionalProperties: Additional properties of type SecretVariablesValue
    """

    class AdditionalProperty(_messages.Message):
      r"""An additional property for a SecretVariablesValue object.

      Fields:
        key: Name of the additional property.
        value: A string attribute.
      """

      key = _messages.StringField(1)
      value = _messages.StringField(2)

    additionalProperties = _messages.MessageField('AdditionalProperty', 1, repeated=True)

  @encoding.MapUnrecognizedFields('additionalProperties')
  class VariablesValue(_messages.Message):
    r"""A map of environment variable names to values.

    Messages:
      AdditionalProperty: An additional property for a VariablesValue object.

    Fields:
      additionalProperties: Additional properties of type VariablesValue
    """

    class AdditionalProperty(_messages.Message):
      r"""An additional property for a VariablesValue object.

      Fields:
        key: Name of the additional property.
        value: A string attribute.
      """

      key = _messages.StringField(1)
      value = _messages.StringField(2)

    additionalProperties = _messages.MessageField('AdditionalProperty', 1, repeated=True)

  encryptedVariables = _messages.MessageField('KMSEnvMap', 1)
  secretVariables = _messages.MessageField('SecretVariablesValue', 2)
  variables = _messages.MessageField('VariablesValue', 3)


class GCS(_messages.Message):
  r"""Represents a Google Cloud Storage volume.

  Fields:
    remotePath: Remote path, either a bucket name or a subdirectory of a
      bucket, e.g.: bucket_name, bucket_name/subdirectory/
  """

  remotePath = _messages.StringField(1)


class InstancePolicy(_messages.Message):
  r"""InstancePolicy describes an instance type and resources attached to each
  VM created by this InstancePolicy.

  Enums:
    ProvisioningModelValueValuesEnum: The provisioning model.

  Fields:
    accelerators: The accelerators attached to each VM instance.
    bootDisk: Boot disk to be created and attached to each VM by this
      InstancePolicy. Boot disk will be deleted when the VM is deleted. Batch
      API now only supports booting from image.
    disks: Non-boot disks to be attached for each VM created by this
      InstancePolicy. New disks will be deleted when the VM is deleted. A non-
      boot disk is a disk that can be of a device with a file system or a raw
      storage drive that is not ready for data storage and accessing.
    machineType: The Compute Engine machine type.
    minCpuPlatform: The minimum CPU platform. See
      https://cloud.google.com/compute/docs/instances/specify-min-cpu-
      platform.
    provisioningModel: The provisioning model.
    reservation: Optional. If not specified (default), VMs will consume any
      applicable reservation. If "NO_RESERVATION" is specified, VMs will not
      consume any reservation. Otherwise, if specified, VMs will consume only
      the specified reservation.
  """

  class ProvisioningModelValueValuesEnum(_messages.Enum):
    r"""The provisioning model.

    Values:
      PROVISIONING_MODEL_UNSPECIFIED: Unspecified.
      STANDARD: Standard VM.
      SPOT: SPOT VM.
      PREEMPTIBLE: Preemptible VM (PVM). Above SPOT VM is the preferable model
        for preemptible VM instances: the old preemptible VM model (indicated
        by this field) is the older model, and has been migrated to use the
        SPOT model as the underlying technology. This old model will still be
        supported.
      RESERVATION_BOUND: Bound to the lifecycle of the reservation in which it
        is provisioned.
      FLEX_START: Instance is provisioned with DWS Flex Start and has limited
        max run duration.
    """
    PROVISIONING_MODEL_UNSPECIFIED = 0
    STANDARD = 1
    SPOT = 2
    PREEMPTIBLE = 3
    RESERVATION_BOUND = 4
    FLEX_START = 5

  accelerators = _messages.MessageField('Accelerator', 1, repeated=True)
  bootDisk = _messages.MessageField('Disk', 2)
  disks = _messages.MessageField('AttachedDisk', 3, repeated=True)
  machineType = _messages.StringField(4)
  minCpuPlatform = _messages.StringField(5)
  provisioningModel = _messages.EnumField('ProvisioningModelValueValuesEnum', 6)
  reservation = _messages.StringField(7)


class InstancePolicyOrTemplate(_messages.Message):
  r"""InstancePolicyOrTemplate lets you define the type of resources to use
  for this job either with an InstancePolicy or an instance template. If
  undefined, Batch picks the type of VM to use and doesn't include optional VM
  resources such as GPUs and extra disks.

  Fields:
    blockProjectSshKeys: Optional. Set this field to `true` if you want Batch
      to block project-level SSH keys from accessing this job's VMs.
      Alternatively, you can configure the job to specify a VM instance
      template that blocks project-level SSH keys. In either case, Batch
      blocks project-level SSH keys while creating the VMs for this job. Batch
      allows project-level SSH keys for a job's VMs only if all the following
      are true: + This field is undefined or set to `false`. + The job's VM
      instance template (if any) doesn't block project-level SSH keys.
      Notably, you can override this behavior by manually updating a VM to
      block or allow project-level SSH keys. For more information about
      blocking project-level SSH keys, see the Compute Engine documentation:
      https://cloud.google.com/compute/docs/connect/restrict-ssh-keys#block-
      keys
    installGpuDrivers: Set this field true if you want Batch to help fetch
      drivers from a third party location and install them for GPUs specified
      in `policy.accelerators` or `instance_template` on your behalf. Default
      is false. For Container-Optimized Image cases, Batch will install the
      accelerator driver following milestones of
      https://cloud.google.com/container-optimized-os/docs/release-notes. For
      non Container-Optimized Image cases, following
      https://github.com/GoogleCloudPlatform/compute-gpu-
      installation/blob/main/linux/install_gpu_driver.py.
    installOpsAgent: Optional. Set this field true if you want Batch to
      install Ops Agent on your behalf. Default is false.
    instanceTemplate: Name of an instance template used to create VMs. Named
      the field as 'instance_template' instead of 'template' to avoid C++
      keyword conflict. Batch only supports global instance templates from the
      same project as the job. You can specify the global instance template as
      a full or partial URL.
    policy: InstancePolicy.
  """

  blockProjectSshKeys = _messages.BooleanField(1)
  installGpuDrivers = _messages.BooleanField(2)
  installOpsAgent = _messages.BooleanField(3)
  instanceTemplate = _messages.StringField(4)
  policy = _messages.MessageField('InstancePolicy', 5)


class InstanceStatus(_messages.Message):
  r"""VM instance status.

  Enums:
    ProvisioningModelValueValuesEnum: The VM instance provisioning model.

  Fields:
    bootDisk: The VM boot disk.
    machineType: The Compute Engine machine type.
    provisioningModel: The VM instance provisioning model.
    taskPack: The max number of tasks can be assigned to this instance type.
  """

  class ProvisioningModelValueValuesEnum(_messages.Enum):
    r"""The VM instance provisioning model.

    Values:
      PROVISIONING_MODEL_UNSPECIFIED: Unspecified.
      STANDARD: Standard VM.
      SPOT: SPOT VM.
      PREEMPTIBLE: Preemptible VM (PVM). Above SPOT VM is the preferable model
        for preemptible VM instances: the old preemptible VM model (indicated
        by this field) is the older model, and has been migrated to use the
        SPOT model as the underlying technology. This old model will still be
        supported.
      RESERVATION_BOUND: Bound to the lifecycle of the reservation in which it
        is provisioned.
      FLEX_START: Instance is provisioned with DWS Flex Start and has limited
        max run duration.
    """
    PROVISIONING_MODEL_UNSPECIFIED = 0
    STANDARD = 1
    SPOT = 2
    PREEMPTIBLE = 3
    RESERVATION_BOUND = 4
    FLEX_START = 5

  bootDisk = _messages.MessageField('Disk', 1)
  machineType = _messages.StringField(2)
  provisioningModel = _messages.EnumField('ProvisioningModelValueValuesEnum', 3)
  taskPack = _messages.IntegerField(4)


class Job(_messages.Message):
  r"""The Cloud Batch Job description.

  Messages:
    LabelsValue: Custom labels to apply to the job and any Cloud Logging [LogE
      ntry](https://cloud.google.com/logging/docs/reference/v2/rest/v2/LogEntr
      y) that it generates. Use labels to group and describe the resources
      they are applied to. Batch automatically applies predefined labels and
      supports multiple `labels` fields for each job, which each let you apply
      custom labels to various resources. Label names that start with "goog-"
      or "google-" are reserved for predefined labels. For more information
      about labels with Batch, see [Organize resources using
      labels](https://cloud.google.com/batch/docs/organize-resources-using-
      labels).

  Fields:
    allocationPolicy: Compute resource allocation for all TaskGroups in the
      Job.
    createTime: Output only. When the Job was created.
    labels: Custom labels to apply to the job and any Cloud Logging [LogEntry]
      (https://cloud.google.com/logging/docs/reference/v2/rest/v2/LogEntry)
      that it generates. Use labels to group and describe the resources they
      are applied to. Batch automatically applies predefined labels and
      supports multiple `labels` fields for each job, which each let you apply
      custom labels to various resources. Label names that start with "goog-"
      or "google-" are reserved for predefined labels. For more information
      about labels with Batch, see [Organize resources using
      labels](https://cloud.google.com/batch/docs/organize-resources-using-
      labels).
    logsPolicy: Log preservation policy for the Job.
    name: Output only. Job name. For example: "projects/123456/locations/us-
      central1/jobs/job01".
    notifications: Notification configurations.
    priority: Priority of the Job. The valid value range is [0, 100). Default
      value is 0. Higher value indicates higher priority. A job with higher
      priority value is more likely to run earlier if all other requirements
      are satisfied.
    status: Output only. Job status. It is read only for users.
    taskGroups: Required. TaskGroups in the Job. Only one TaskGroup is
      supported now.
    uid: Output only. A system generated unique ID for the Job.
    updateTime: Output only. The last time the Job was updated.
  """

  @encoding.MapUnrecognizedFields('additionalProperties')
  class LabelsValue(_messages.Message):
    r"""Custom labels to apply to the job and any Cloud Logging [LogEntry](htt
    ps://cloud.google.com/logging/docs/reference/v2/rest/v2/LogEntry) that it
    generates. Use labels to group and describe the resources they are applied
    to. Batch automatically applies predefined labels and supports multiple
    `labels` fields for each job, which each let you apply custom labels to
    various resources. Label names that start with "goog-" or "google-" are
    reserved for predefined labels. For more information about labels with
    Batch, see [Organize resources using
    labels](https://cloud.google.com/batch/docs/organize-resources-using-
    labels).

    Messages:
      AdditionalProperty: An additional property for a LabelsValue object.

    Fields:
      additionalProperties: Additional properties of type LabelsValue
    """

    class AdditionalProperty(_messages.Message):
      r"""An additional property for a LabelsValue object.

      Fields:
        key: Name of the additional property.
        value: A string attribute.
      """

      key = _messages.StringField(1)
      value = _messages.StringField(2)

    additionalProperties = _messages.MessageField('AdditionalProperty', 1, repeated=True)

  allocationPolicy = _messages.MessageField('AllocationPolicy', 1)
  createTime = _messages.StringField(2)
  labels = _messages.MessageField('LabelsValue', 3)
  logsPolicy = _messages.MessageField('LogsPolicy', 4)
  name = _messages.StringField(5)
  notifications = _messages.MessageField('JobNotification', 6, repeated=True)
  priority = _messages.IntegerField(7)
  status = _messages.MessageField('JobStatus', 8)
  taskGroups = _messages.MessageField('TaskGroup', 9, repeated=True)
  uid = _messages.StringField(10)
  updateTime = _messages.StringField(11)


class JobNotification(_messages.Message):
  r"""Notification configurations.

  Fields:
    message: The attribute requirements of messages to be sent to this Pub/Sub
      topic. Without this field, no message will be sent.
    pubsubTopic: The Pub/Sub topic where notifications for the job, like state
      changes, will be published. If undefined, no Pub/Sub notifications are
      sent for this job. Specify the topic using the following format:
      `projects/{project}/topics/{topic}`. Notably, if you want to specify a
      Pub/Sub topic that is in a different project than the job, your
      administrator must grant your project's Batch service agent permission
      to publish to that topic. For more information about configuring Pub/Sub
      notifications for a job, see https://cloud.google.com/batch/docs/enable-
      notifications.
  """

  message = _messages.MessageField('Message', 1)
  pubsubTopic = _messages.StringField(2)


class JobStatus(_messages.Message):
  r"""Job status.

  Enums:
    StateValueValuesEnum: Job state

  Messages:
    TaskGroupsValue: Aggregated task status for each TaskGroup in the Job. The
      map key is TaskGroup ID.

  Fields:
    runDuration: The duration of time that the Job spent in status RUNNING.
    state: Job state
    statusEvents: Job status events
    taskGroups: Aggregated task status for each TaskGroup in the Job. The map
      key is TaskGroup ID.
  """

  class StateValueValuesEnum(_messages.Enum):
    r"""Job state

    Values:
      STATE_UNSPECIFIED: Job state unspecified.
      QUEUED: Job is admitted (validated and persisted) and waiting for
        resources.
      SCHEDULED: Job is scheduled to run as soon as resource allocation is
        ready. The resource allocation may happen at a later time but with a
        high chance to succeed.
      RUNNING: Resource allocation has been successful. At least one Task in
        the Job is RUNNING.
      SUCCEEDED: All Tasks in the Job have finished successfully.
      FAILED: At least one Task in the Job has failed.
      DELETION_IN_PROGRESS: The Job will be deleted, but has not been deleted
        yet. Typically this is because resources used by the Job are still
        being cleaned up.
      CANCELLATION_IN_PROGRESS: The Job cancellation is in progress, this is
        because the resources used by the Job are still being cleaned up.
      CANCELLED: The Job has been cancelled, the task executions were stopped
        and the resources were cleaned up.
    """
    STATE_UNSPECIFIED = 0
    QUEUED = 1
    SCHEDULED = 2
    RUNNING = 3
    SUCCEEDED = 4
    FAILED = 5
    DELETION_IN_PROGRESS = 6
    CANCELLATION_IN_PROGRESS = 7
    CANCELLED = 8

  @encoding.MapUnrecognizedFields('additionalProperties')
  class TaskGroupsValue(_messages.Message):
    r"""Aggregated task status for each TaskGroup in the Job. The map key is
    TaskGroup ID.

    Messages:
      AdditionalProperty: An additional property for a TaskGroupsValue object.

    Fields:
      additionalProperties: Additional properties of type TaskGroupsValue
    """

    class AdditionalProperty(_messages.Message):
      r"""An additional property for a TaskGroupsValue object.

      Fields:
        key: Name of the additional property.
        value: A TaskGroupStatus attribute.
      """

      key = _messages.StringField(1)
      value = _messages.MessageField('TaskGroupStatus', 2)

    additionalProperties = _messages.MessageField('AdditionalProperty', 1, repeated=True)

  runDuration = _messages.StringField(1)
  state = _messages.EnumField('StateValueValuesEnum', 2)
  statusEvents = _messages.MessageField('StatusEvent', 3, repeated=True)
  taskGroups = _messages.MessageField('TaskGroupsValue', 4)


class KMSEnvMap(_messages.Message):
  r"""A KMSEnvMap object.

  Fields:
    cipherText: The value of the cipherText response from the `encrypt`
      method.
    keyName: The name of the KMS key that will be used to decrypt the cipher
      text.
  """

  cipherText = _messages.StringField(1)
  keyName = _messages.StringField(2)


class LifecyclePolicy(_messages.Message):
  r"""LifecyclePolicy describes how to deal with task failures based on
  different conditions.

  Enums:
    ActionValueValuesEnum: Action to execute when ActionCondition is true.
      When RETRY_TASK is specified, we will retry failed tasks if we notice
      any exit code match and fail tasks if no match is found. Likewise, when
      FAIL_TASK is specified, we will fail tasks if we notice any exit code
      match and retry tasks if no match is found.

  Fields:
    action: Action to execute when ActionCondition is true. When RETRY_TASK is
      specified, we will retry failed tasks if we notice any exit code match
      and fail tasks if no match is found. Likewise, when FAIL_TASK is
      specified, we will fail tasks if we notice any exit code match and retry
      tasks if no match is found.
    actionCondition: Conditions that decide why a task failure is dealt with a
      specific action.
  """

  class ActionValueValuesEnum(_messages.Enum):
    r"""Action to execute when ActionCondition is true. When RETRY_TASK is
    specified, we will retry failed tasks if we notice any exit code match and
    fail tasks if no match is found. Likewise, when FAIL_TASK is specified, we
    will fail tasks if we notice any exit code match and retry tasks if no
    match is found.

    Values:
      ACTION_UNSPECIFIED: Action unspecified.
      RETRY_TASK: Action that tasks in the group will be scheduled to re-
        execute.
      FAIL_TASK: Action that tasks in the group will be stopped immediately.
    """
    ACTION_UNSPECIFIED = 0
    RETRY_TASK = 1
    FAIL_TASK = 2

  action = _messages.EnumField('ActionValueValuesEnum', 1)
  actionCondition = _messages.MessageField('ActionCondition', 2)


class ListJobsResponse(_messages.Message):
  r"""ListJob Response.

  Fields:
    jobs: Jobs.
    nextPageToken: Next page token.
    unreachable: Locations that could not be reached.
  """

  jobs = _messages.MessageField('Job', 1, repeated=True)
  nextPageToken = _messages.StringField(2)
  unreachable = _messages.StringField(3, repeated=True)


class ListLocationsResponse(_messages.Message):
  r"""The response message for Locations.ListLocations.

  Fields:
    locations: A list of locations that matches the specified filter in the
      request.
    nextPageToken: The standard List next-page token.
  """

  locations = _messages.MessageField('Location', 1, repeated=True)
  nextPageToken = _messages.StringField(2)


class ListOperationsResponse(_messages.Message):
  r"""The response message for Operations.ListOperations.

  Fields:
    nextPageToken: The standard List next-page token.
    operations: A list of operations that matches the specified filter in the
      request.
    unreachable: Unordered list. Unreachable resources. Populated when the
      request sets `ListOperationsRequest.return_partial_success` and reads
      across collections. For example, when attempting to list all resources
      across all supported locations.
  """

  nextPageToken = _messages.StringField(1)
  operations = _messages.MessageField('Operation', 2, repeated=True)
  unreachable = _messages.StringField(3, repeated=True)


class ListTasksResponse(_messages.Message):
  r"""ListTasks Response.

  Fields:
    nextPageToken: Next page token.
    tasks: Tasks.
    unreachable: Locations that could not be reached.
  """

  nextPageToken = _messages.StringField(1)
  tasks = _messages.MessageField('Task', 2, repeated=True)
  unreachable = _messages.StringField(3, repeated=True)


class Location(_messages.Message):
  r"""A resource that represents a Google Cloud location.

  Messages:
    LabelsValue: Cross-service attributes for the location. For example
      {"cloud.googleapis.com/region": "us-east1"}
    MetadataValue: Service-specific metadata. For example the available
      capacity at the given location.

  Fields:
    displayName: The friendly name for this location, typically a nearby city
      name. For example, "Tokyo".
    labels: Cross-service attributes for the location. For example
      {"cloud.googleapis.com/region": "us-east1"}
    locationId: The canonical id for this location. For example: `"us-east1"`.
    metadata: Service-specific metadata. For example the available capacity at
      the given location.
    name: Resource name for the location, which may vary between
      implementations. For example: `"projects/example-project/locations/us-
      east1"`
  """

  @encoding.MapUnrecognizedFields('additionalProperties')
  class LabelsValue(_messages.Message):
    r"""Cross-service attributes for the location. For example
    {"cloud.googleapis.com/region": "us-east1"}

    Messages:
      AdditionalProperty: An additional property for a LabelsValue object.

    Fields:
      additionalProperties: Additional properties of type LabelsValue
    """

    class AdditionalProperty(_messages.Message):
      r"""An additional property for a LabelsValue object.

      Fields:
        key: Name of the additional property.
        value: A string attribute.
      """

      key = _messages.StringField(1)
      value = _messages.StringField(2)

    additionalProperties = _messages.MessageField('AdditionalProperty', 1, repeated=True)

  @encoding.MapUnrecognizedFields('additionalProperties')
  class MetadataValue(_messages.Message):
    r"""Service-specific metadata. For example the available capacity at the
    given location.

    Messages:
      AdditionalProperty: An additional property for a MetadataValue object.

    Fields:
      additionalProperties: Properties of the object. Contains field @type
        with type URL.
    """

    class AdditionalProperty(_messages.Message):
      r"""An additional property for a MetadataValue object.

      Fields:
        key: Name of the additional property.
        value: A extra_types.JsonValue attribute.
      """

      key = _messages.StringField(1)
      value = _messages.MessageField('extra_types.JsonValue', 2)

    additionalProperties = _messages.MessageField('AdditionalProperty', 1, repeated=True)

  displayName = _messages.StringField(1)
  labels = _messages.MessageField('LabelsValue', 2)
  locationId = _messages.StringField(3)
  metadata = _messages.MessageField('MetadataValue', 4)
  name = _messages.StringField(5)


class LocationPolicy(_messages.Message):
  r"""A LocationPolicy object.

  Fields:
    allowedLocations: A list of allowed location names represented by internal
      URLs. Each location can be a region or a zone. Only one region or
      multiple zones in one region is supported now. For example,
      ["regions/us-central1"] allow VMs in any zones in region us-central1.
      ["zones/us-central1-a", "zones/us-central1-c"] only allow VMs in zones
      us-central1-a and us-central1-c. Mixing locations from different regions
      would cause errors. For example, ["regions/us-central1", "zones/us-
      central1-a", "zones/us-central1-b", "zones/us-west1-a"] contains
      locations from two distinct regions: us-central1 and us-west1. This
      combination will trigger an error.
  """

  allowedLocations = _messages.StringField(1, repeated=True)


class LogsPolicy(_messages.Message):
  r"""LogsPolicy describes if and how a job's logs are preserved. Logs include
  information that is automatically written by the Batch service agent and any
  information that you configured the job's runnables to write to the `stdout`
  or `stderr` streams.

  Enums:
    DestinationValueValuesEnum: If and where logs should be saved.

  Fields:
    cloudLoggingOption: Optional. When `destination` is set to
      `CLOUD_LOGGING`, you can optionally set this field to configure
      additional settings for Cloud Logging.
    destination: If and where logs should be saved.
    logsPath: When `destination` is set to `PATH`, you must set this field to
      the path where you want logs to be saved. This path can point to a local
      directory on the VM or (if congifured) a directory under the mount path
      of any Cloud Storage bucket, network file system (NFS), or writable
      persistent disk that is mounted to the job. For example, if the job has
      a bucket with `mountPath` set to `/mnt/disks/my-bucket`, you can write
      logs to the root directory of the `remotePath` of that bucket by setting
      this field to `/mnt/disks/my-bucket/`.
  """

  class DestinationValueValuesEnum(_messages.Enum):
    r"""If and where logs should be saved.

    Values:
      DESTINATION_UNSPECIFIED: (Default) Logs are not preserved.
      CLOUD_LOGGING: Logs are streamed to Cloud Logging. Optionally, you can
        configure additional settings in the `cloudLoggingOption` field.
      PATH: Logs are saved to the file path specified in the `logsPath` field.
    """
    DESTINATION_UNSPECIFIED = 0
    CLOUD_LOGGING = 1
    PATH = 2

  cloudLoggingOption = _messages.MessageField('CloudLoggingOption', 1)
  destination = _messages.EnumField('DestinationValueValuesEnum', 2)
  logsPath = _messages.StringField(3)


class Message(_messages.Message):
  r"""Message details. Describe the conditions under which messages will be
  sent. If no attribute is defined, no message will be sent by default. One
  message should specify either the job or the task level attributes, but not
  both. For example, job level: JOB_STATE_CHANGED and/or a specified
  new_job_state; task level: TASK_STATE_CHANGED and/or a specified
  new_task_state.

  Enums:
    NewJobStateValueValuesEnum: The new job state.
    NewTaskStateValueValuesEnum: The new task state.
    TypeValueValuesEnum: The message type.

  Fields:
    newJobState: The new job state.
    newTaskState: The new task state.
    type: The message type.
  """

  class NewJobStateValueValuesEnum(_messages.Enum):
    r"""The new job state.

    Values:
      STATE_UNSPECIFIED: Job state unspecified.
      QUEUED: Job is admitted (validated and persisted) and waiting for
        resources.
      SCHEDULED: Job is scheduled to run as soon as resource allocation is
        ready. The resource allocation may happen at a later time but with a
        high chance to succeed.
      RUNNING: Resource allocation has been successful. At least one Task in
        the Job is RUNNING.
      SUCCEEDED: All Tasks in the Job have finished successfully.
      FAILED: At least one Task in the Job has failed.
      DELETION_IN_PROGRESS: The Job will be deleted, but has not been deleted
        yet. Typically this is because resources used by the Job are still
        being cleaned up.
      CANCELLATION_IN_PROGRESS: The Job cancellation is in progress, this is
        because the resources used by the Job are still being cleaned up.
      CANCELLED: The Job has been cancelled, the task executions were stopped
        and the resources were cleaned up.
    """
    STATE_UNSPECIFIED = 0
    QUEUED = 1
    SCHEDULED = 2
    RUNNING = 3
    SUCCEEDED = 4
    FAILED = 5
    DELETION_IN_PROGRESS = 6
    CANCELLATION_IN_PROGRESS = 7
    CANCELLED = 8

  class NewTaskStateValueValuesEnum(_messages.Enum):
    r"""The new task state.

    Values:
      STATE_UNSPECIFIED: Unknown state.
      PENDING: The Task is created and waiting for resources.
      ASSIGNED: The Task is assigned to at least one VM.
      RUNNING: The Task is running.
      FAILED: The Task has failed.
      SUCCEEDED: The Task has succeeded.
      UNEXECUTED: The Task has not been executed when the Job finishes.
    """
    STATE_UNSPECIFIED = 0
    PENDING = 1
    ASSIGNED = 2
    RUNNING = 3
    FAILED = 4
    SUCCEEDED = 5
    UNEXECUTED = 6

  class TypeValueValuesEnum(_messages.Enum):
    r"""The message type.

    Values:
      TYPE_UNSPECIFIED: Unspecified.
      JOB_STATE_CHANGED: Notify users that the job state has changed.
      TASK_STATE_CHANGED: Notify users that the task state has changed.
    """
    TYPE_UNSPECIFIED = 0
    JOB_STATE_CHANGED = 1
    TASK_STATE_CHANGED = 2

  newJobState = _messages.EnumField('NewJobStateValueValuesEnum', 1)
  newTaskState = _messages.EnumField('NewTaskStateValueValuesEnum', 2)
  type = _messages.EnumField('TypeValueValuesEnum', 3)


class NFS(_messages.Message):
  r"""Represents an NFS volume.

  Fields:
    remotePath: Remote source path exported from the NFS, e.g., "/share".
    server: The IP address of the NFS.
  """

  remotePath = _messages.StringField(1)
  server = _messages.StringField(2)


class NetworkInterface(_messages.Message):
  r"""A network interface.

  Fields:
    network: The URL of an existing network resource. You can specify the
      network as a full or partial URL. For example, the following are all
      valid URLs: * https://www.googleapis.com/compute/v1/projects/{project}/g
      lobal/networks/{network} * projects/{project}/global/networks/{network}
      * global/networks/{network}
    noExternalIpAddress: Default is false (with an external IP address).
      Required if no external public IP address is attached to the VM. If no
      external public IP address, additional configuration is required to
      allow the VM to access Google Services. See
      https://cloud.google.com/vpc/docs/configure-private-google-access and
      https://cloud.google.com/nat/docs/gce-example#create-nat for more
      information.
    subnetwork: The URL of an existing subnetwork resource in the network. You
      can specify the subnetwork as a full or partial URL. For example, the
      following are all valid URLs: * https://www.googleapis.com/compute/v1/pr
      ojects/{project}/regions/{region}/subnetworks/{subnetwork} *
      projects/{project}/regions/{region}/subnetworks/{subnetwork} *
      regions/{region}/subnetworks/{subnetwork}
  """

  network = _messages.StringField(1)
  noExternalIpAddress = _messages.BooleanField(2)
  subnetwork = _messages.StringField(3)


class NetworkPolicy(_messages.Message):
  r"""NetworkPolicy describes VM instance network configurations.

  Fields:
    networkInterfaces: Network configurations.
  """

  networkInterfaces = _messages.MessageField('NetworkInterface', 1, repeated=True)


class Operation(_messages.Message):
  r"""This resource represents a long-running operation that is the result of
  a network API call.

  Messages:
    MetadataValue: Service-specific metadata associated with the operation. It
      typically contains progress information and common metadata such as
      create time. Some services might not provide such metadata. Any method
      that returns a long-running operation should document the metadata type,
      if any.
    ResponseValue: The normal, successful response of the operation. If the
      original method returns no data on success, such as `Delete`, the
      response is `google.protobuf.Empty`. If the original method is standard
      `Get`/`Create`/`Update`, the response should be the resource. For other
      methods, the response should have the type `XxxResponse`, where `Xxx` is
      the original method name. For example, if the original method name is
      `TakeSnapshot()`, the inferred response type is `TakeSnapshotResponse`.

  Fields:
    done: If the value is `false`, it means the operation is still in
      progress. If `true`, the operation is completed, and either `error` or
      `response` is available.
    error: The error result of the operation in case of failure or
      cancellation.
    metadata: Service-specific metadata associated with the operation. It
      typically contains progress information and common metadata such as
      create time. Some services might not provide such metadata. Any method
      that returns a long-running operation should document the metadata type,
      if any.
    name: The server-assigned name, which is only unique within the same
      service that originally returns it. If you use the default HTTP mapping,
      the `name` should be a resource name ending with
      `operations/{unique_id}`.
    response: The normal, successful response of the operation. If the
      original method returns no data on success, such as `Delete`, the
      response is `google.protobuf.Empty`. If the original method is standard
      `Get`/`Create`/`Update`, the response should be the resource. For other
      methods, the response should have the type `XxxResponse`, where `Xxx` is
      the original method name. For example, if the original method name is
      `TakeSnapshot()`, the inferred response type is `TakeSnapshotResponse`.
  """

  @encoding.MapUnrecognizedFields('additionalProperties')
  class MetadataValue(_messages.Message):
    r"""Service-specific metadata associated with the operation. It typically
    contains progress information and common metadata such as create time.
    Some services might not provide such metadata. Any method that returns a
    long-running operation should document the metadata type, if any.

    Messages:
      AdditionalProperty: An additional property for a MetadataValue object.

    Fields:
      additionalProperties: Properties of the object. Contains field @type
        with type URL.
    """

    class AdditionalProperty(_messages.Message):
      r"""An additional property for a MetadataValue object.

      Fields:
        key: Name of the additional property.
        value: A extra_types.JsonValue attribute.
      """

      key = _messages.StringField(1)
      value = _messages.MessageField('extra_types.JsonValue', 2)

    additionalProperties = _messages.MessageField('AdditionalProperty', 1, repeated=True)

  @encoding.MapUnrecognizedFields('additionalProperties')
  class ResponseValue(_messages.Message):
    r"""The normal, successful response of the operation. If the original
    method returns no data on success, such as `Delete`, the response is
    `google.protobuf.Empty`. If the original method is standard
    `Get`/`Create`/`Update`, the response should be the resource. For other
    methods, the response should have the type `XxxResponse`, where `Xxx` is
    the original method name. For example, if the original method name is
    `TakeSnapshot()`, the inferred response type is `TakeSnapshotResponse`.

    Messages:
      AdditionalProperty: An additional property for a ResponseValue object.

    Fields:
      additionalProperties: Properties of the object. Contains field @type
        with type URL.
    """

    class AdditionalProperty(_messages.Message):
      r"""An additional property for a ResponseValue object.

      Fields:
        key: Name of the additional property.
        value: A extra_types.JsonValue attribute.
      """

      key = _messages.StringField(1)
      value = _messages.MessageField('extra_types.JsonValue', 2)

    additionalProperties = _messages.MessageField('AdditionalProperty', 1, repeated=True)

  done = _messages.BooleanField(1)
  error = _messages.MessageField('Status', 2)
  metadata = _messages.MessageField('MetadataValue', 3)
  name = _messages.StringField(4)
  response = _messages.MessageField('ResponseValue', 5)


class OperationMetadata(_messages.Message):
  r"""Represents the metadata of the long-running operation.

  Fields:
    apiVersion: Output only. API version used to start the operation.
    createTime: Output only. The time the operation was created.
    endTime: Output only. The time the operation finished running.
    requestedCancellation: Output only. Identifies whether the user has
      requested cancellation of the operation. Operations that have
      successfully been cancelled have google.longrunning.Operation.error
      value with a google.rpc.Status.code of 1, corresponding to
      `Code.CANCELLED`.
    statusMessage: Output only. Human-readable status of the operation, if
      any.
    target: Output only. Server-defined resource path for the target of the
      operation.
    verb: Output only. Name of the verb executed by the operation.
  """

  apiVersion = _messages.StringField(1)
  createTime = _messages.StringField(2)
  endTime = _messages.StringField(3)
  requestedCancellation = _messages.BooleanField(4)
  statusMessage = _messages.StringField(5)
  target = _messages.StringField(6)
  verb = _messages.StringField(7)


class PlacementPolicy(_messages.Message):
  r"""PlacementPolicy describes a group placement policy for the VMs
  controlled by this AllocationPolicy.

  Fields:
    collocation: UNSPECIFIED vs. COLLOCATED (default UNSPECIFIED). Use
      COLLOCATED when you want VMs to be located close to each other for low
      network latency between the VMs. No placement policy will be generated
      when collocation is UNSPECIFIED.
    maxDistance: When specified, causes the job to fail if more than
      max_distance logical switches are required between VMs. Batch uses the
      most compact possible placement of VMs even when max_distance is not
      specified. An explicit max_distance makes that level of compactness a
      strict requirement. Not yet implemented
  """

  collocation = _messages.StringField(1)
  maxDistance = _messages.IntegerField(2)


class Runnable(_messages.Message):
  r"""Runnable describes instructions for executing a specific script or
  container as part of a Task.

  Messages:
    LabelsValue: Labels for this Runnable.

  Fields:
    alwaysRun: By default, after a Runnable fails, no further Runnable are
      executed. This flag indicates that this Runnable must be run even if the
      Task has already failed. This is useful for Runnables that copy output
      files off of the VM or for debugging. The always_run flag does not
      override the Task's overall max_run_duration. If the max_run_duration
      has expired then no further Runnables will execute, not even always_run
      Runnables.
    background: Normally, a runnable that doesn't exit causes its task to
      fail. However, you can set this field to `true` to configure a
      background runnable. Background runnables are allowed continue running
      in the background while the task executes subsequent runnables. For
      example, background runnables are useful for providing services to other
      runnables or providing debugging-support tools like SSH servers.
      Specifically, background runnables are killed automatically (if they
      have not already exited) a short time after all foreground runnables
      have completed. Even though this is likely to result in a non-zero exit
      status for the background runnable, these automatic kills are not
      treated as task failures.
    barrier: Barrier runnable.
    container: Container runnable.
    displayName: Optional. DisplayName is an optional field that can be
      provided by the caller. If provided, it will be used in logs and other
      outputs to identify the script, making it easier for users to understand
      the logs. If not provided the index of the runnable will be used for
      outputs.
    environment: Environment variables for this Runnable (overrides variables
      set for the whole Task or TaskGroup).
    ignoreExitStatus: Normally, a runnable that returns a non-zero exit status
      fails and causes the task to fail. However, you can set this field to
      `true` to allow the task to continue executing its other runnables even
      if this runnable fails.
    labels: Labels for this Runnable.
    script: Script runnable.
    timeout: Timeout for this Runnable.
  """

  @encoding.MapUnrecognizedFields('additionalProperties')
  class LabelsValue(_messages.Message):
    r"""Labels for this Runnable.

    Messages:
      AdditionalProperty: An additional property for a LabelsValue object.

    Fields:
      additionalProperties: Additional properties of type LabelsValue
    """

    class AdditionalProperty(_messages.Message):
      r"""An additional property for a LabelsValue object.

      Fields:
        key: Name of the additional property.
        value: A string attribute.
      """

      key = _messages.StringField(1)
      value = _messages.StringField(2)

    additionalProperties = _messages.MessageField('AdditionalProperty', 1, repeated=True)

  alwaysRun = _messages.BooleanField(1)
  background = _messages.BooleanField(2)
  barrier = _messages.MessageField('Barrier', 3)
  container = _messages.MessageField('Container', 4)
  displayName = _messages.StringField(5)
  environment = _messages.MessageField('Environment', 6)
  ignoreExitStatus = _messages.BooleanField(7)
  labels = _messages.MessageField('LabelsValue', 8)
  script = _messages.MessageField('Script', 9)
  timeout = _messages.StringField(10)


class Script(_messages.Message):
  r"""Script runnable.

  Fields:
    path: The path to a script file that is accessible from the host VM(s).
      Unless the script file supports the default `#!/bin/sh` shell
      interpreter, you must specify an interpreter by including a [shebang
      line](https://en.wikipedia.org/wiki/Shebang_(Unix) as the first line of
      the file. For example, to execute the script using bash, include
      `#!/bin/bash` as the first line of the file. Alternatively, to execute
      the script using Python3, include `#!/usr/bin/env python3` as the first
      line of the file.
    text: The text for a script. Unless the script text supports the default
      `#!/bin/sh` shell interpreter, you must specify an interpreter by
      including a [shebang line](https://en.wikipedia.org/wiki/Shebang_(Unix)
      at the beginning of the text. For example, to execute the script using
      bash, include `#!/bin/bash\n` at the beginning of the text.
      Alternatively, to execute the script using Python3, include
      `#!/usr/bin/env python3\n` at the beginning of the text.
  """

  path = _messages.StringField(1)
  text = _messages.StringField(2)


class ServiceAccount(_messages.Message):
  r"""Carries information about a Google Cloud service account.

  Fields:
    email: Email address of the service account.
    scopes: List of scopes to be enabled for this service account.
  """

  email = _messages.StringField(1)
  scopes = _messages.StringField(2, repeated=True)


class StandardQueryParameters(_messages.Message):
  r"""Query parameters accepted by all methods.

  Enums:
    FXgafvValueValuesEnum: V1 error format.
    AltValueValuesEnum: Data format for response.

  Fields:
    f__xgafv: V1 error format.
    access_token: OAuth access token.
    alt: Data format for response.
    callback: JSONP
    fields: Selector specifying which fields to include in a partial response.
    key: API key. Your API key identifies your project and provides you with
      API access, quota, and reports. Required unless you provide an OAuth 2.0
      token.
    oauth_token: OAuth 2.0 token for the current user.
    prettyPrint: Returns response with indentations and line breaks.
    quotaUser: Available to use for quota purposes for server-side
      applications. Can be any arbitrary string assigned to a user, but should
      not exceed 40 characters.
    trace: A tracing token of the form "token:<tokenid>" to include in api
      requests.
    uploadType: Legacy upload protocol for media (e.g. "media", "multipart").
    upload_protocol: Upload protocol for media (e.g. "raw", "multipart").
  """

  class AltValueValuesEnum(_messages.Enum):
    r"""Data format for response.

    Values:
      json: Responses with Content-Type of application/json
      media: Media download with context-dependent Content-Type
      proto: Responses with Content-Type of application/x-protobuf
    """
    json = 0
    media = 1
    proto = 2

  class FXgafvValueValuesEnum(_messages.Enum):
    r"""V1 error format.

    Values:
      _1: v1 error format
      _2: v2 error format
    """
    _1 = 0
    _2 = 1

  f__xgafv = _messages.EnumField('FXgafvValueValuesEnum', 1)
  access_token = _messages.StringField(2)
  alt = _messages.EnumField('AltValueValuesEnum', 3, default='json')
  callback = _messages.StringField(4)
  fields = _messages.StringField(5)
  key = _messages.StringField(6)
  oauth_token = _messages.StringField(7)
  prettyPrint = _messages.BooleanField(8, default=True)
  quotaUser = _messages.StringField(9)
  trace = _messages.StringField(10)
  uploadType = _messages.StringField(11)
  upload_protocol = _messages.StringField(12)


class Status(_messages.Message):
  r"""The `Status` type defines a logical error model that is suitable for
  different programming environments, including REST APIs and RPC APIs. It is
  used by [gRPC](https://github.com/grpc). Each `Status` message contains
  three pieces of data: error code, error message, and error details. You can
  find out more about this error model and how to work with it in the [API
  Design Guide](https://cloud.google.com/apis/design/errors).

  Messages:
    DetailsValueListEntry: A DetailsValueListEntry object.

  Fields:
    code: The status code, which should be an enum value of google.rpc.Code.
    details: A list of messages that carry the error details. There is a
      common set of message types for APIs to use.
    message: A developer-facing error message, which should be in English. Any
      user-facing error message should be localized and sent in the
      google.rpc.Status.details field, or localized by the client.
  """

  @encoding.MapUnrecognizedFields('additionalProperties')
  class DetailsValueListEntry(_messages.Message):
    r"""A DetailsValueListEntry object.

    Messages:
      AdditionalProperty: An additional property for a DetailsValueListEntry
        object.

    Fields:
      additionalProperties: Properties of the object. Contains field @type
        with type URL.
    """

    class AdditionalProperty(_messages.Message):
      r"""An additional property for a DetailsValueListEntry object.

      Fields:
        key: Name of the additional property.
        value: A extra_types.JsonValue attribute.
      """

      key = _messages.StringField(1)
      value = _messages.MessageField('extra_types.JsonValue', 2)

    additionalProperties = _messages.MessageField('AdditionalProperty', 1, repeated=True)

  code = _messages.IntegerField(1, variant=_messages.Variant.INT32)
  details = _messages.MessageField('DetailsValueListEntry', 2, repeated=True)
  message = _messages.StringField(3)


class StatusEvent(_messages.Message):
  r"""Status event.

  Enums:
    TaskStateValueValuesEnum: Task State. This field is only defined for task-
      level status events.

  Fields:
    description: Description of the event.
    eventTime: The time this event occurred.
    taskExecution: Task Execution. This field is only defined for task-level
      status events where the task fails.
    taskState: Task State. This field is only defined for task-level status
      events.
    type: Type of the event.
  """

  class TaskStateValueValuesEnum(_messages.Enum):
    r"""Task State. This field is only defined for task-level status events.

    Values:
      STATE_UNSPECIFIED: Unknown state.
      PENDING: The Task is created and waiting for resources.
      ASSIGNED: The Task is assigned to at least one VM.
      RUNNING: The Task is running.
      FAILED: The Task has failed.
      SUCCEEDED: The Task has succeeded.
      UNEXECUTED: The Task has not been executed when the Job finishes.
    """
    STATE_UNSPECIFIED = 0
    PENDING = 1
    ASSIGNED = 2
    RUNNING = 3
    FAILED = 4
    SUCCEEDED = 5
    UNEXECUTED = 6

  description = _messages.StringField(1)
  eventTime = _messages.StringField(2)
  taskExecution = _messages.MessageField('TaskExecution', 3)
  taskState = _messages.EnumField('TaskStateValueValuesEnum', 4)
  type = _messages.StringField(5)


class Task(_messages.Message):
  r"""A Cloud Batch task.

  Fields:
    name: Task name. The name is generated from the parent TaskGroup name and
      'id' field. For example: "projects/123456/locations/us-
      west1/jobs/job01/taskGroups/group01/tasks/task01".
    status: Task Status.
  """

  name = _messages.StringField(1)
  status = _messages.MessageField('TaskStatus', 2)


class TaskExecution(_messages.Message):
  r"""This Task Execution field includes detail information for task execution
  procedures, based on StatusEvent types.

  Fields:
    exitCode: The exit code of a finished task. If the task succeeded, the
      exit code will be 0. If the task failed but not due to the following
      reasons, the exit code will be 50000. Otherwise, it can be from
      different sources: * Batch known failures:
      https://cloud.google.com/batch/docs/troubleshooting#reserved-exit-codes.
      * Batch runnable execution failures; you can rely on Batch logs to
      further diagnose: https://cloud.google.com/batch/docs/analyze-job-using-
      logs. If there are multiple runnables failures, Batch only exposes the
      first error.
  """

  exitCode = _messages.IntegerField(1, variant=_messages.Variant.INT32)


class TaskGroup(_messages.Message):
  r"""A TaskGroup defines one or more Tasks that all share the same TaskSpec.

  Enums:
    SchedulingPolicyValueValuesEnum: Scheduling policy for Tasks in the
      TaskGroup. The default value is AS_SOON_AS_POSSIBLE.

  Fields:
    name: Output only. TaskGroup name. The system generates this field based
      on parent Job name. For example: "projects/123456/locations/us-
      west1/jobs/job01/taskGroups/group01".
    parallelism: Max number of tasks that can run in parallel. Default to
      min(task_count, parallel tasks per job limit). See: [Job
      Limits](https://cloud.google.com/batch/quotas#job_limits). Field
      parallelism must be 1 if the scheduling_policy is IN_ORDER.
    permissiveSsh: When true, Batch will configure SSH to allow passwordless
      login between VMs running the Batch tasks in the same TaskGroup.
    requireHostsFile: When true, Batch will populate a file with a list of all
      VMs assigned to the TaskGroup and set the BATCH_HOSTS_FILE environment
      variable to the path of that file. Defaults to false. The host file
      supports up to 1000 VMs.
    runAsNonRoot: Optional. If not set or set to false, Batch uses the root
      user to execute runnables. If set to true, Batch runs the runnables
      using a non-root user. Currently, the non-root user Batch used is
      generated by OS Login. For more information, see [About OS
      Login](https://cloud.google.com/compute/docs/oslogin).
    schedulingPolicy: Scheduling policy for Tasks in the TaskGroup. The
      default value is AS_SOON_AS_POSSIBLE.
    taskCount: Number of Tasks in the TaskGroup. Default is 1.
    taskCountPerNode: Max number of tasks that can be run on a VM at the same
      time. If not specified, the system will decide a value based on
      available compute resources on a VM and task requirements.
    taskEnvironments: An array of environment variable mappings, which are
      passed to Tasks with matching indices. If task_environments is used then
      task_count should not be specified in the request (and will be ignored).
      Task count will be the length of task_environments. Tasks get a
      BATCH_TASK_INDEX and BATCH_TASK_COUNT environment variable, in addition
      to any environment variables set in task_environments, specifying the
      number of Tasks in the Task's parent TaskGroup, and the specific Task's
      index in the TaskGroup (0 through BATCH_TASK_COUNT - 1).
    taskSpec: Required. Tasks in the group share the same task spec.
  """

  class SchedulingPolicyValueValuesEnum(_messages.Enum):
    r"""Scheduling policy for Tasks in the TaskGroup. The default value is
    AS_SOON_AS_POSSIBLE.

    Values:
      SCHEDULING_POLICY_UNSPECIFIED: Unspecified.
      AS_SOON_AS_POSSIBLE: Run Tasks as soon as resources are available. Tasks
        might be executed in parallel depending on parallelism and task_count
        values.
      IN_ORDER: Run Tasks sequentially with increased task index.
    """
    SCHEDULING_POLICY_UNSPECIFIED = 0
    AS_SOON_AS_POSSIBLE = 1
    IN_ORDER = 2

  name = _messages.StringField(1)
  parallelism = _messages.IntegerField(2)
  permissiveSsh = _messages.BooleanField(3)
  requireHostsFile = _messages.BooleanField(4)
  runAsNonRoot = _messages.BooleanField(5)
  schedulingPolicy = _messages.EnumField('SchedulingPolicyValueValuesEnum', 6)
  taskCount = _messages.IntegerField(7)
  taskCountPerNode = _messages.IntegerField(8)
  taskEnvironments = _messages.MessageField('Environment', 9, repeated=True)
  taskSpec = _messages.MessageField('TaskSpec', 10)


class TaskGroupStatus(_messages.Message):
  r"""Aggregated task status for a TaskGroup.

  Messages:
    CountsValue: Count of task in each state in the TaskGroup. The map key is
      task state name.

  Fields:
    counts: Count of task in each state in the TaskGroup. The map key is task
      state name.
    instances: Status of instances allocated for the TaskGroup.
  """

  @encoding.MapUnrecognizedFields('additionalProperties')
  class CountsValue(_messages.Message):
    r"""Count of task in each state in the TaskGroup. The map key is task
    state name.

    Messages:
      AdditionalProperty: An additional property for a CountsValue object.

    Fields:
      additionalProperties: Additional properties of type CountsValue
    """

    class AdditionalProperty(_messages.Message):
      r"""An additional property for a CountsValue object.

      Fields:
        key: Name of the additional property.
        value: A string attribute.
      """

      key = _messages.StringField(1)
      value = _messages.IntegerField(2)

    additionalProperties = _messages.MessageField('AdditionalProperty', 1, repeated=True)

  counts = _messages.MessageField('CountsValue', 1)
  instances = _messages.MessageField('InstanceStatus', 2, repeated=True)


class TaskSpec(_messages.Message):
  r"""Spec of a task

  Messages:
    EnvironmentsValue: Deprecated: please use environment(non-plural) instead.

  Fields:
    computeResource: ComputeResource requirements.
    environment: Environment variables to set before running the Task.
    environments: Deprecated: please use environment(non-plural) instead.
    lifecyclePolicies: Lifecycle management schema when any task in a task
      group is failed. Currently we only support one lifecycle policy. When
      the lifecycle policy condition is met, the action in the policy will
      execute. If task execution result does not meet with the defined
      lifecycle policy, we consider it as the default policy. Default policy
      means if the exit code is 0, exit task. If task ends with non-zero exit
      code, retry the task with max_retry_count.
    maxRetryCount: Maximum number of retries on failures. The default, 0,
      which means never retry. The valid value range is [0, 10].
    maxRunDuration: Maximum duration the task should run before being
      automatically retried (if enabled) or automatically failed. Format the
      value of this field as a time limit in seconds followed by `s`-for
      example, `3600s` for 1 hour. The field accepts any value between 0 and
      the maximum listed for the `Duration` field type at
      https://protobuf.dev/reference/protobuf/google.protobuf/#duration;
      however, the actual maximum run time for a job will be limited to the
      maximum run time for a job listed at
      https://cloud.google.com/batch/quotas#max-job-duration.
    runnables: Required. The sequence of one or more runnables (executable
      scripts, executable containers, and/or barriers) for each task in this
      task group to run. Each task runs this list of runnables in order. For a
      task to succeed, all of its script and container runnables each must
      meet at least one of the following conditions: + The runnable exited
      with a zero status. + The runnable didn't finish, but you enabled its
      `background` subfield. + The runnable exited with a non-zero status, but
      you enabled its `ignore_exit_status` subfield.
    volumes: Volumes to mount before running Tasks using this TaskSpec.
  """

  @encoding.MapUnrecognizedFields('additionalProperties')
  class EnvironmentsValue(_messages.Message):
    r"""Deprecated: please use environment(non-plural) instead.

    Messages:
      AdditionalProperty: An additional property for a EnvironmentsValue
        object.

    Fields:
      additionalProperties: Additional properties of type EnvironmentsValue
    """

    class AdditionalProperty(_messages.Message):
      r"""An additional property for a EnvironmentsValue object.

      Fields:
        key: Name of the additional property.
        value: A string attribute.
      """

      key = _messages.StringField(1)
      value = _messages.StringField(2)

    additionalProperties = _messages.MessageField('AdditionalProperty', 1, repeated=True)

  computeResource = _messages.MessageField('ComputeResource', 1)
  environment = _messages.MessageField('Environment', 2)
  environments = _messages.MessageField('EnvironmentsValue', 3)
  lifecyclePolicies = _messages.MessageField('LifecyclePolicy', 4, repeated=True)
  maxRetryCount = _messages.IntegerField(5, variant=_messages.Variant.INT32)
  maxRunDuration = _messages.StringField(6)
  runnables = _messages.MessageField('Runnable', 7, repeated=True)
  volumes = _messages.MessageField('Volume', 8, repeated=True)


class TaskStatus(_messages.Message):
  r"""Status of a task.

  Enums:
    StateValueValuesEnum: Task state.

  Fields:
    state: Task state.
    statusEvents: Detailed info about why the state is reached.
  """

  class StateValueValuesEnum(_messages.Enum):
    r"""Task state.

    Values:
      STATE_UNSPECIFIED: Unknown state.
      PENDING: The Task is created and waiting for resources.
      ASSIGNED: The Task is assigned to at least one VM.
      RUNNING: The Task is running.
      FAILED: The Task has failed.
      SUCCEEDED: The Task has succeeded.
      UNEXECUTED: The Task has not been executed when the Job finishes.
    """
    STATE_UNSPECIFIED = 0
    PENDING = 1
    ASSIGNED = 2
    RUNNING = 3
    FAILED = 4
    SUCCEEDED = 5
    UNEXECUTED = 6

  state = _messages.EnumField('StateValueValuesEnum', 1)
  statusEvents = _messages.MessageField('StatusEvent', 2, repeated=True)


class Volume(_messages.Message):
  r"""Volume describes a volume and parameters for it to be mounted to a VM.

  Fields:
    deviceName: Device name of an attached disk volume, which should align
      with a device_name specified by
      job.allocation_policy.instances[0].policy.disks[i].device_name or
      defined by the given instance template in
      job.allocation_policy.instances[0].instance_template.
    gcs: A Google Cloud Storage (GCS) volume.
    mountOptions: Mount options vary based on the type of storage volume: *
      For a Cloud Storage bucket, all the mount options provided by the
      [`gcsfuse` tool](https://cloud.google.com/storage/docs/gcsfuse-cli) are
      supported. * For an existing persistent disk, all mount options provided
      by the [`mount` command](https://man7.org/linux/man-
      pages/man8/mount.8.html) except writing are supported. This is due to
      restrictions of [multi-writer
      mode](https://cloud.google.com/compute/docs/disks/sharing-disks-between-
      vms). * For any other disk or a Network File System (NFS), all the mount
      options provided by the `mount` command are supported.
    mountPath: The mount path for the volume, e.g. /mnt/disks/share.
    nfs: A Network File System (NFS) volume. For example, a Filestore file
      share.
  """

  deviceName = _messages.StringField(1)
  gcs = _messages.MessageField('GCS', 2)
  mountOptions = _messages.StringField(3, repeated=True)
  mountPath = _messages.StringField(4)
  nfs = _messages.MessageField('NFS', 5)


encoding.AddCustomJsonFieldMapping(
    StandardQueryParameters, 'f__xgafv', '$.xgafv')
encoding.AddCustomJsonEnumMapping(
    StandardQueryParameters.FXgafvValueValuesEnum, '_1', '1')
encoding.AddCustomJsonEnumMapping(
    StandardQueryParameters.FXgafvValueValuesEnum, '_2', '2')



# -*- coding: utf-8 -*- #
# Copyright 2020 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Command to create a datastream private connection."""


from googlecloudsdk.api_lib.datastream import private_connections
from googlecloudsdk.api_lib.datastream import util
from googlecloudsdk.calliope import base
from googlecloudsdk.command_lib.datastream import flags
from googlecloudsdk.command_lib.datastream import resource_args
from googlecloudsdk.command_lib.datastream.private_connections import flags as pc_flags
from googlecloudsdk.command_lib.util.concepts import concept_parsers
from googlecloudsdk.command_lib.util.concepts import presentation_specs


DESCRIPTION = ('Create a Datastream private connection')
EXAMPLES = """\
    To create a privateConnection with VPC Peering called 'my-privateConnection', run:

        $ {command} my-privateConnection --location=us-central1 --display-name=my-privateConnection --vpc=vpc-example --subnet=10.0.0.0/29

    To create a privateConnection with PSC Interface called 'my-privateConnection', run:

        $ {command} my-privateConnection --location=us-central1 --display-name=my-privateConnection --network-attachment=network-attachment-example
   """
EXAMPLES_BETA = """\
    To create a privateConnection with VPC Peering called 'my-privateConnection', run:

        $ {command} my-privateConnection --location=us-central1 --display-name=my-privateConnection --vpc-name=vpc-example --subnet=10.0.0.0/29


   """


@base.RegionalEndpointsSupported
@base.DefaultUniverseOnly
@base.ReleaseTracks(base.ReleaseTrack.GA)
class Create(base.Command):
  """Create a Datastream private connection."""
  detailed_help = {'DESCRIPTION': DESCRIPTION, 'EXAMPLES': EXAMPLES}

  @staticmethod
  def CommonArgs(parser, release_track):
    """Common arguments for all release tracks.

    Args:
      parser: An argparse parser that you can use to add arguments that go on
        the command line after this command. Positional arguments are allowed.
      release_track: Some arguments are added based on the command release
        track.
    """
    resource_args.AddPrivateConnectionResourceArg(parser, 'to create')
    pc_flags.AddDisplayNameFlag(parser)
    # pc_flags.AddNetworkAttachmentFlag(parser)
    pc_flags.AddValidateOnlyFlag(parser)
    flags.AddLabelsCreateFlags(parser)

    config_group = parser.add_group(mutex=True, required=True)

    vpc_peering_group = config_group.add_group(
        help='Arguments for VPC Peering configuration.'
    )
    vpc_peering_group.add_argument(
        '--subnet',
        help="""A free subnet for peering. (CIDR of /29).""",
        required=True,
    )

    # Add VPC resource arg inside the VPC Peering group
    vpc_field_name = 'vpc'
    if release_track == base.ReleaseTrack.BETA:
      vpc_field_name = 'vpc-name'

    vpc_spec = presentation_specs.ResourcePresentationSpec(
        '--%s' % vpc_field_name,
        resource_args.GetVpcResourceSpec(),
        'Resource ID of the VPC network to peer with.',
        required=True,
    )  # Ensure VPC is required within this group
    concept_parsers.ConceptParser([vpc_spec]).AddToParser(vpc_peering_group)

    # --- PSC Interface Group ---
    if release_track == base.ReleaseTrack.GA:
      psc_group = config_group.add_group(
          help='Arguments for Private Service Connect Interface configuration.'
      )
      pc_flags.AddNetworkAttachmentFlag(psc_group)

  @staticmethod
  def Args(parser):
    """Args is called by calliope to gather arguments for this command."""
    Create.CommonArgs(parser, base.ReleaseTrack.GA)

  def Run(self, args):
    """Create a Datastream private connection.

    Args:
      args: argparse.Namespace, The arguments that this command was invoked
        with.

    Returns:
      A dict object representing the operations resource describing the create
      operation if the create was successful.
    """
    private_connection_ref = args.CONCEPTS.private_connection.Parse()
    parent_ref = private_connection_ref.Parent().RelativeName()

    pc_client = private_connections.PrivateConnectionsClient(
        location=private_connection_ref.locationsId
    )
    result_operation = pc_client.Create(
        parent_ref, private_connection_ref.privateConnectionsId,
        self.ReleaseTrack(), args)

    client = util.GetClientInstance(location=private_connection_ref.locationsId)
    messages = util.GetMessagesModule()
    resource_parser = util.GetResourceParser()

    operation_ref = resource_parser.Create(
        'datastream.projects.locations.operations',
        operationsId=result_operation.name,
        projectsId=private_connection_ref.projectsId,
        locationsId=private_connection_ref.locationsId)

    return client.projects_locations_operations.Get(
        messages.DatastreamProjectsLocationsOperationsGetRequest(
            name=operation_ref.operationsId))


@base.Deprecate(
    is_removed=False,
    warning=(
        'Datastream beta version is deprecated. Please use`gcloud '
        'datastream private-connections create` command instead.'
    ),
)
@base.RegionalEndpointsSupported
@base.DefaultUniverseOnly
@base.ReleaseTracks(base.ReleaseTrack.BETA)
class CreateBeta(Create):
  """Create a Datastream private connection."""
  detailed_help = {'DESCRIPTION': DESCRIPTION, 'EXAMPLES': EXAMPLES_BETA}

  @staticmethod
  def Args(parser):
    """Args is called by calliope to gather arguments for this command."""
    Create.CommonArgs(parser, base.ReleaseTrack.BETA)



# -*- coding: utf-8 -*- #
# Copyright 2017 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Flags and helpers for the compute security policies commands."""

from googlecloudsdk.calliope import arg_parsers
from googlecloudsdk.command_lib.compute import completers as compute_completers
from googlecloudsdk.command_lib.compute import flags as compute_flags
from googlecloudsdk.command_lib.util import completers


class GlobalSecurityPoliciesCompleter(compute_completers.ListCommandCompleter):

  def __init__(self, **kwargs):
    super(GlobalSecurityPoliciesCompleter, self).__init__(
        collection='compute.securityPolicies',
        list_command='compute security-policies list --uri',
        **kwargs)


class RegionalSecurityPoliciesCompleter(compute_completers.ListCommandCompleter
                                       ):

  def __init__(self, **kwargs):
    super(RegionalSecurityPoliciesCompleter, self).__init__(
        collection='compute.regionSecurityPolicies',
        list_command=('compute security-policies list '
                      '--filter=region:* --uri'),
        **kwargs)


class SecurityPoliciesCompleter(completers.MultiResourceCompleter):

  def __init__(self, **kwargs):
    super(SecurityPoliciesCompleter, self).__init__(
        completers=[
            GlobalSecurityPoliciesCompleter, RegionalSecurityPoliciesCompleter
        ],
        **kwargs)


def SecurityPolicyArgument(required=True, plural=False):
  return compute_flags.ResourceArgument(
      resource_name='security policy',
      completer=SecurityPoliciesCompleter,
      plural=plural,
      custom_plural='security policies',
      required=required,
      global_collection='compute.securityPolicies')


def SecurityPolicyRegionalArgument(required=True, plural=False):
  return compute_flags.ResourceArgument(
      resource_name='security policy',
      completer=SecurityPoliciesCompleter,
      plural=plural,
      custom_plural='security policies',
      required=required,
      regional_collection='compute.regionSecurityPolicies')


def SecurityPolicyMultiScopeArgument(required=True, plural=False):
  return compute_flags.ResourceArgument(
      resource_name='security policy',
      completer=SecurityPoliciesCompleter,
      plural=plural,
      custom_plural='security policies',
      required=required,
      global_collection='compute.securityPolicies',
      regional_collection='compute.regionSecurityPolicies')


def SecurityPolicyArgumentForTargetResource(resource, required=False):
  return compute_flags.ResourceArgument(
      resource_name='security policy',
      name='--security-policy',
      completer=SecurityPoliciesCompleter,
      plural=False,
      required=required,
      global_collection='compute.securityPolicies',
      short_help=('The security policy that will be set for this {0}.'.format(
          resource)))


def SecurityPolicyRegionalArgumentForTargetResource(resource, required=False):
  return compute_flags.ResourceArgument(
      resource_name='security policy',
      name='--security-policy',
      completer=RegionalSecurityPoliciesCompleter,
      plural=False,
      required=required,
      regional_collection='compute.regionSecurityPolicies',
      short_help=(
          ('The security policy that will be set for this {0}. To remove the '
           'policy from this {0} set the policy to an empty string.'
          ).format(resource)))


def SecurityPolicyMultiScopeArgumentForTargetResource(
    resource,
    required=False,
    region_hidden=False,
    scope_flags_usage=compute_flags.ScopeFlagsUsage
    .GENERATE_DEDICATED_SCOPE_FLAGS,
    short_help_text=None):
  return compute_flags.ResourceArgument(
      resource_name='security policy',
      name='--security-policy',
      completer=SecurityPoliciesCompleter,
      plural=False,
      required=required,
      global_collection='compute.securityPolicies',
      regional_collection='compute.regionSecurityPolicies',
      region_hidden=region_hidden,
      short_help=(
          (short_help_text or
           'The security policy that will be set for this {0}. To remove the '
           'policy from this {0} set the policy to an empty string.'
          ).format(resource)),
      scope_flags_usage=scope_flags_usage)


def EdgeSecurityPolicyArgumentForTargetResource(resource, required=False):
  return compute_flags.ResourceArgument(
      resource_name='security policy',
      name='--edge-security-policy',
      completer=SecurityPoliciesCompleter,
      plural=False,
      required=required,
      global_collection='compute.securityPolicies',
      short_help=(
          ('The edge security policy that will be set for this {0}. To remove '
           'the policy from this {0} set the policy to an empty string.'
          ).format(resource)))


def SecurityPolicyArgumentForRules(required=False):
  return compute_flags.ResourceArgument(
      resource_name='security policy',
      name='--security-policy',
      completer=SecurityPoliciesCompleter,
      plural=False,
      required=required,
      global_collection='compute.securityPolicies',
      short_help='The security policy that this rule belongs to.')


def SecurityPolicyMultiScopeArgumentForRules(required=False):
  return compute_flags.ResourceArgument(
      resource_name='security policy',
      name='--security-policy',
      completer=SecurityPoliciesCompleter,
      plural=False,
      required=required,
      global_collection='compute.securityPolicies',
      regional_collection='compute.regionSecurityPolicies',
      region_hidden=True,
      scope_flags_usage=compute_flags.ScopeFlagsUsage.USE_EXISTING_SCOPE_FLAGS,
      short_help='The security policy that this rule belongs to.')


def AddCloudArmorAdaptiveProtection(parser, required=False):
  """Adds the cloud armor adaptive protection arguments to the argparse."""
  parser.add_argument(
      '--enable-layer7-ddos-defense',
      action='store_true',
      default=None,
      required=required,
      help=('Whether to enable Cloud Armor Layer 7 DDoS Defense Adaptive '
            'Protection.'))
  parser.add_argument(
      '--layer7-ddos-defense-rule-visibility',
      choices=['STANDARD', 'PREMIUM'],
      type=lambda x: x.upper(),
      required=required,
      metavar='VISIBILITY_TYPE',
      help=('The visibility type indicates whether the rules are opaque or '
            'transparent.'))


def AddCloudArmorAdaptiveProtectionAutoDeploy(parser):
  """Adds the cloud armor adaptive protection's auto-deploy arguments to the argparse."""
  parser.add_argument(
      '--layer7-ddos-defense-auto-deploy-load-threshold',
      type=float,
      help=(
          "Load threshold above which Adaptive Protection's auto-deploy takes actions"
      ))
  parser.add_argument(
      '--layer7-ddos-defense-auto-deploy-confidence-threshold',
      type=float,
      help=(
          "Confidence threshold above which Adaptive Protection's auto-deploy takes actions"
      ))
  parser.add_argument(
      '--layer7-ddos-defense-auto-deploy-impacted-baseline-threshold',
      type=float,
      help=(
          "Impacted baseline threshold below which Adaptive Protection's auto-deploy takes actions"
      ))
  parser.add_argument(
      '--layer7-ddos-defense-auto-deploy-expiration-sec',
      type=int,
      help=(
          "Duration over which Adaptive Protection's auto-deployed actions last"
      ))


def AddAdvancedOptions(parser, required=False, enable_large_body_size=False):
  """Adds the cloud armor advanced options arguments to the argparse."""
  parser.add_argument(
      '--json-parsing',
      choices=['DISABLED', 'STANDARD', 'STANDARD_WITH_GRAPHQL'],
      type=lambda x: x.upper(),
      required=required,
      help=('The JSON parsing behavior for this rule. '
            'Must be one of the following values: '
            '[DISABLED, STANDARD, STANDARD_WITH_GRAPHQL].'))

  parser.add_argument(
      '--json-custom-content-types',
      type=arg_parsers.ArgList(),
      metavar='CONTENT_TYPE',
      help="""\
      A comma-separated list of custom Content-Type header values to apply JSON
      parsing for preconfigured WAF rules. Only applicable when JSON parsing is
      enabled, like ``--json-parsing=STANDARD''. When configuring a Content-Type
      header value, only the type/subtype needs to be specified, and the
      parameters should be excluded.
      """)

  parser.add_argument(
      '--log-level',
      choices=['NORMAL', 'VERBOSE'],
      type=lambda x: x.upper(),
      required=required,
      help='The level of detail to display for WAF logging.')

  if enable_large_body_size:
    parser.add_argument(
        '--request-body-inspection-size',
        choices=['8KB', '16KB', '32KB', '48KB', '64KB'],
        type=lambda x: x.upper(),
        required=required,
        help='Maximum request body inspection size.',
    )

  parser.add_argument(
      '--user-ip-request-headers',
      type=arg_parsers.ArgList(),
      metavar='USER_IP_REQUEST_HEADER',
      help="""\
      A comma-separated list of request header names to use for resolving the
      caller's user IP address.
      """)


def AddDdosProtectionConfigWithAdvancedPreview(parser, required=False):
  """Adds the cloud armor DDoS protection config arguments to the argparse."""
  parser.add_argument(
      '--network-ddos-protection',
      choices=['STANDARD', 'ADVANCED', 'ADVANCED_PREVIEW'],
      type=lambda x: x.upper(),
      required=required,
      help=(
          'The DDoS protection level for network load balancing and instances '
          'with external IPs'
      ),
  )


def AddDdosProtectionConfigOld(parser, required=False):
  """Adds the cloud armor DDoS protection config arguments to the argparse."""
  parser.add_argument(
      '--ddos-protection',
      choices=['STANDARD', 'ADVANCED', 'ADVANCED_PREVIEW'],
      type=lambda x: x.upper(),
      required=required,
      help=(
          'The DDoS protection level for network load balancing and instances '
          'with external IPs'
      ),
  )


def AddNetworkDdosAdaptiveProtection(parser, required=False):
  """Adds the Cloud Armor Network DDoS adaptive protection arguments."""
  parser.add_argument(
      '--network-ddos-adaptive-protection',
      choices=['DISABLED', 'ENABLED', 'PREVIEW'],
      type=lambda x: x.upper(),
      required=required,
      hidden=True,
      help=(
          'The DDoS adaptive protection level for network load balancing and'
          ' instances with external IPs'
      ),
  )


def AddNetworkDdosImpactedBaselineThreshold(parser, required=False):
  """Adds the Cloud Armor Network DDoS impacted baseline threshold argument."""
  impacted_baseline_threshold_group = parser.add_mutually_exclusive_group(
      required=required, hidden=True
  )
  impacted_baseline_threshold_group.add_argument(
      '--network-ddos-impacted-baseline-threshold',
      type=float,
      hidden=True,
      help="""\
      DDoS Protection for Network Load Balancers (and VMs with public IPs)
      builds DDoS mitigations that minimize collateral damage. It quantifies
      this as the fraction of a non-abuse baseline that's inadvertently blocked.
      Rules whose collateral damage exceeds ddosImpactedBaselineThreshold will
      not be deployed. Using a lower value will prioritize keeping collateral
      damage low, possibly at the cost of its effectiveness in rate limiting
      some or all of the attack. It should typically be unset, so Advanced DDoS
      (and Adaptive Protection) uses the best mitigation it can find. Setting
      the threshold is advised if there are logs for false positive detections
      with high collateral damage, and will cause Advanced DDoS to attempt to
      find a less aggressive rule that satisfies the constraint. If a suitable
      rule cannot be found, the system falls back to either no mitigation for
      smaller attacks or broader network throttles for larger ones.
      """,
  )
  impacted_baseline_threshold_group.add_argument(
      '--clear-network-ddos-impacted-baseline-threshold',
      action='store_true',
      hidden=True,
      help=('If provided, clears the Network DDoS impacted baseline threshold '
            'from the security policy.'),
  )


def AddRecaptchaOptions(parser, required=False):
  """Adds the cloud armor reCAPTCHA options arguments to the argparse."""
  parser.add_argument(
      '--recaptcha-redirect-site-key',
      required=required,
      help="""\
      The reCAPTCHA site key to be used for rules using the ``redirect'' action
      and the ``google-recaptcha'' redirect type under the security policy.
      """)



# -*- coding: utf-8 -*- #
# Copyright 2022 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Shared resource args for the Dataplex surface."""


from googlecloudsdk.calliope.concepts import concepts
from googlecloudsdk.calliope.concepts import deps
from googlecloudsdk.command_lib.util.apis import yaml_data
from googlecloudsdk.command_lib.util.concepts import concept_parsers
from googlecloudsdk.core import properties


GENERATE_ID = '@%!#DATAPLEX_GENERATE_UUID@%!#'


def GetProjectSpec():
  """Gets Project spec."""
  return concepts.ResourceSpec(
      'dataplex.projects',
      resource_name='projects',
      projectsId=concepts.DEFAULT_PROJECT_ATTRIBUTE_CONFIG,
  )


def GetLakeResourceSpec():
  """Gets Lake resource spec."""
  return concepts.ResourceSpec(
      'dataplex.projects.locations.lakes',
      resource_name='lakes',
      projectsId=concepts.DEFAULT_PROJECT_ATTRIBUTE_CONFIG,
      locationsId=LocationAttributeConfig(),
      lakesId=LakeAttributeConfig(),
  )


def GetZoneResourceSpec():
  """Gets Zone resource spec."""
  return concepts.ResourceSpec(
      'dataplex.projects.locations.lakes.zones',
      resource_name='zones',
      projectsId=concepts.DEFAULT_PROJECT_ATTRIBUTE_CONFIG,
      locationsId=LocationAttributeConfig(),
      lakesId=LakeAttributeConfig(),
      zonesId=ZoneAttributeConfig(),
  )


def GetAssetResourceSpec():
  """Gets Asset resource spec."""
  return concepts.ResourceSpec(
      'dataplex.projects.locations.lakes.zones.assets',
      resource_name='assets',
      projectsId=concepts.DEFAULT_PROJECT_ATTRIBUTE_CONFIG,
      locationsId=LocationAttributeConfig(),
      lakesId=LakeAttributeConfig(),
      zonesId=ZoneAttributeConfig(),
      assetsId=AssetAttributeConfig(),
  )


def GetContentitemResourceSpec():
  """Gets Content resource spec."""
  return concepts.ResourceSpec(
      'dataplex.projects.locations.lakes.contentitems',
      resource_name='content',
      projectsId=concepts.DEFAULT_PROJECT_ATTRIBUTE_CONFIG,
      locationsId=LocationAttributeConfig(),
      lakesId=LakeAttributeConfig(),
      contentitemsId=ContentAttributeConfig(),
  )


def GetTaskResourceSpec():
  """Gets Task resource spec."""
  return concepts.ResourceSpec(
      'dataplex.projects.locations.lakes.tasks',
      resource_name='tasks',
      projectsId=concepts.DEFAULT_PROJECT_ATTRIBUTE_CONFIG,
      locationsId=LocationAttributeConfig(),
      lakesId=LakeAttributeConfig(),
      tasksId=TaskAttributeConfig(),
  )


def GetEnvironmentResourceSpec():
  """Gets Environment resource spec."""
  return concepts.ResourceSpec(
      'dataplex.projects.locations.lakes.environments',
      resource_name='environments',
      projectsId=concepts.DEFAULT_PROJECT_ATTRIBUTE_CONFIG,
      locationsId=LocationAttributeConfig(),
      lakesId=LakeAttributeConfig(),
      environmentsId=EnvironmentAttributeConfig(),
  )


def GetDatascanResourceSpec():
  """Gets Datascan resource spec."""
  return concepts.ResourceSpec(
      'dataplex.projects.locations.dataScans',
      resource_name='datascan',
      projectsId=concepts.DEFAULT_PROJECT_ATTRIBUTE_CONFIG,
      locationsId=LocationAttributeConfig(),
      dataScansId=DatascanAttributeConfig(),
  )


def GetDataTaxonomyResourceSpec():
  """Gets DataTaxonomy resource spec."""
  return concepts.ResourceSpec(
      'dataplex.projects.locations.dataTaxonomies',
      resource_name='data taxonomy',
      projectsId=concepts.DEFAULT_PROJECT_ATTRIBUTE_CONFIG,
      locationsId=LocationAttributeConfig(),
      dataTaxonomiesId=DataTaxonomyAttributeConfig(),
  )


def GetDataAttributeBindingResourceSpec():
  """Gets DataAttributeBinding resource spec."""
  return concepts.ResourceSpec(
      'dataplex.projects.locations.dataAttributeBindings',
      resource_name='data attribute binding',
      projectsId=concepts.DEFAULT_PROJECT_ATTRIBUTE_CONFIG,
      locationsId=LocationAttributeConfig(),
      dataAttributeBindingsId=DataAttributeBindingAttributeConfig(),
  )


def GetDataAttributeResourceSpec():
  """Gets Data Attribute resource spec."""
  return concepts.ResourceSpec(
      'dataplex.projects.locations.dataTaxonomies.attributes',
      resource_name='data attribute',
      projectsId=concepts.DEFAULT_PROJECT_ATTRIBUTE_CONFIG,
      locationsId=LocationAttributeConfig(),
      dataTaxonomiesId=DataTaxonomyAttributeConfig(),
      attributesId=DataAttributeConfig(),
  )


def GetDataplexEntryGroupResourceSpec():
  """Gets Entry Group resource spec."""
  return concepts.ResourceSpec(
      'dataplex.projects.locations.entryGroups',
      resource_name='entry group',
      projectsId=concepts.DEFAULT_PROJECT_ATTRIBUTE_CONFIG,
      locationsId=LocationAttributeConfig(),
      entryGroupsId=EntryGroupAttributeConfig(),
  )


def GetDataplexAspectTypeResourceSpec():
  """Gets Aspect Type resource spec."""
  return concepts.ResourceSpec(
      'dataplex.projects.locations.aspectTypes',
      resource_name='aspect type',
      projectsId=concepts.DEFAULT_PROJECT_ATTRIBUTE_CONFIG,
      locationsId=LocationAttributeConfig(),
      aspectTypesId=AspectTypeAttributeConfig(),
  )


def GetDataplexEntryTypeResourceSpec():
  """Gets Entry Type resource spec."""
  return concepts.ResourceSpec(
      'dataplex.projects.locations.entryTypes',
      resource_name='entry type',
      projectsId=concepts.DEFAULT_PROJECT_ATTRIBUTE_CONFIG,
      locationsId=LocationAttributeConfig(),
      entryTypesId=EntryTypeAttributeConfig(),
  )


def GetEntryTypeResourceSpec():
  """Gets EntryType resource spec."""
  return concepts.ResourceSpec(
      'dataplex.projects.locations.entryTypes',
      resource_name='entry type',
      projectsId=EntryTypeProjectAttributeConfig(),
      locationsId=EntryTypeLocationAttributeConfig(),
      entryTypesId=EntryTypeConfig(),
  )


def GetDataplexEntryLinkResourceSpec():
  """Gets Entry Link resource spec."""
  return concepts.ResourceSpec(
      'dataplex.projects.locations.entryGroups.entryLinks',
      resource_name='entry link',
      projectsId=concepts.DEFAULT_PROJECT_ATTRIBUTE_CONFIG,
      locationsId=LocationAttributeConfig(),
      entryGroupsId=EntryGroupAttributeConfig(),
      entryLinksId=EntryLinkAttributeConfig(),
  )


def GetGovernanceRuleResourceSpec():
  """Gets GovernanceRule resource spec."""
  return concepts.ResourceSpec(
      'dataplex.projects.locations.governanceRules',
      resource_name='governance rule',
      projectsId=concepts.DEFAULT_PROJECT_ATTRIBUTE_CONFIG,
      locationsId=LocationAttributeConfig(),
      governanceRulesId=GovernanceRuleAttributeConfig(),
  )


def GetGlossaryResourceSpec():
  """Gets Glossary resource spec."""
  return concepts.ResourceSpec(
      'dataplex.projects.locations.glossaries',
      resource_name='glossary',
      projectsId=concepts.DEFAULT_PROJECT_ATTRIBUTE_CONFIG,
      locationsId=LocationAttributeConfig(),
      glossariesId=GlossaryAttributeConfig(),
  )


def GetGlossaryCategoryResourceSpec():
  """Gets Glossary Category resource spec."""
  return concepts.ResourceSpec(
      'dataplex.projects.locations.glossaries.categories',
      resource_name='glossary category',
      projectsId=concepts.DEFAULT_PROJECT_ATTRIBUTE_CONFIG,
      locationsId=LocationAttributeConfig(),
      glossariesId=GlossaryAttributeConfig(),
      categoriesId=GlossaryCategoryAttributeConfig(),
  )


def GetGlossaryTermResourceSpec():
  """Gets Glossary Term resource spec."""
  return concepts.ResourceSpec(
      'dataplex.projects.locations.glossaries.terms',
      resource_name='glossary term',
      projectsId=concepts.DEFAULT_PROJECT_ATTRIBUTE_CONFIG,
      locationsId=LocationAttributeConfig(),
      glossariesId=GlossaryAttributeConfig(),
      termsId=GlossaryTermAttributeConfig(),
  )


def GetMetadataJobResourceSpec():
  """Gets Metadata Job resource spec."""
  return concepts.ResourceSpec(
      'dataplex.projects.locations.metadataJobs',
      resource_name='metadata job',
      projectsId=concepts.DEFAULT_PROJECT_ATTRIBUTE_CONFIG,
      locationsId=LocationAttributeConfig(),
      metadataJobsId=MetadataJobAttributeConfig(),
  )


def GetDataProductResourceSpec():
  """Gets Data Product resource spec."""
  return concepts.ResourceSpec(
      'dataplex.projects.locations.dataProducts',
      resource_name='data product',
      projectsId=concepts.DEFAULT_PROJECT_ATTRIBUTE_CONFIG,
      locationsId=LocationAttributeConfig(),
      dataProductsId=DataProductAttributeConfig(),
  )


def GetEncryptionConfigResourceSpec():
  """Gets EncryptionConfig resource spec."""
  return concepts.ResourceSpec(
      'dataplex.organizations.locations.encryptionConfigs',
      resource_name='encryption config',
      organizationsId=OrganizationAttributeConfig(),
      locationsId=LocationAttributeConfig(),
      encryptionConfigsId=EncryptionConfigAttributeConfig(),
  )


def EntryTypeProjectAttributeConfig():
  return concepts.ResourceParameterAttributeConfig(
      name='entry-type-project',
      help_text='The project of the EntryType resource.',
  )


def EntryTypeLocationAttributeConfig():
  return concepts.ResourceParameterAttributeConfig(
      name='entry-type-location',
      help_text='The location of the EntryType resource.',
  )


def LocationAttributeConfig():
  return concepts.ResourceParameterAttributeConfig(
      name='location',
      fallthroughs=[
          deps.PropertyFallthrough(properties.FromString('dataplex/location'))
      ],
      help_text='The location of the Dataplex resource.',
  )


def LakeAttributeConfig():
  return concepts.ResourceParameterAttributeConfig(
      name='lake', help_text='The identifier of the Dataplex lake resource.'
  )


def ZoneAttributeConfig():
  return concepts.ResourceParameterAttributeConfig(
      name='zone', help_text='The identifier of the Dataplex zone resource.'
  )


def AssetAttributeConfig():
  return concepts.ResourceParameterAttributeConfig(
      name='asset', help_text='The identifier of the Dataplex asset resource.'
  )


def ContentAttributeConfig():
  return concepts.ResourceParameterAttributeConfig(
      name='content', help_text='The name of the {resource} to use.'
  )


def EnvironmentAttributeConfig():
  return concepts.ResourceParameterAttributeConfig(
      name='environment', help_text='The name of {resource} to use.'
  )


def DataTaxonomyAttributeConfig():
  return concepts.ResourceParameterAttributeConfig(
      name='data_taxonomy', help_text='The name of {resource} to use.'
  )


def DataAttributeConfig():
  return concepts.ResourceParameterAttributeConfig(
      name='data_attribute', help_text='The name of {resource} to use.'
  )


def DataAttributeBindingAttributeConfig():
  return concepts.ResourceParameterAttributeConfig(
      name='data_attribute_binding', help_text='The name of {resource} to use.'
  )


def EntryGroupAttributeConfig():
  return concepts.ResourceParameterAttributeConfig(
      name='entry-group', help_text='The name of {resource} to use.'
  )


def AspectTypeAttributeConfig():
  return concepts.ResourceParameterAttributeConfig(
      name='aspect_type', help_text='The name of {resource} to use.'
  )


def EntryTypeAttributeConfig():
  return concepts.ResourceParameterAttributeConfig(
      name='entry_type', help_text='The name of {resource} to use.'
  )


def EntryLinkAttributeConfig():
  return concepts.ResourceParameterAttributeConfig(
      name='entry-link', help_text='The name of {resource} to use.'
  )


def DatascanAttributeConfig():
  return concepts.ResourceParameterAttributeConfig(
      name='dataScans', help_text='The name of {resource} to use.'
  )


def EntryTypeConfig():
  return concepts.ResourceParameterAttributeConfig(
      name='entry_type', help_text='The name of {resource} to use.'
  )


def GovernanceRuleAttributeConfig():
  return concepts.ResourceParameterAttributeConfig(
      name='governance_rule', help_text='The name of {resource} to use.'
  )


def GlossaryAttributeConfig():
  return concepts.ResourceParameterAttributeConfig(
      name='glossary', help_text='The name of {resource} to use.'
  )


def GlossaryCategoryAttributeConfig():
  return concepts.ResourceParameterAttributeConfig(
      name='glossary_category', help_text='The name of {resource} to use.'
  )


def GlossaryTermAttributeConfig():
  return concepts.ResourceParameterAttributeConfig(
      name='glossary_term', help_text='The name of {resource} to use.'
  )


def MetadataJobAttributeConfig():
  return concepts.ResourceParameterAttributeConfig(
      name='metadata_job',
      # Adding invalid job_id to keep job resource in the right format,
      # this invalid value will be removed if no job_id is specified from
      # the input and the underlaying client would generate a valid one.
      fallthroughs=[
          deps.ValueFallthrough(
              GENERATE_ID,
              hint='job ID is optional and will be generated if not specified',
          )
      ],
      help_text='The name of {resource} to use.',
  )


def EncryptionConfigAttributeConfig():
  return concepts.ResourceParameterAttributeConfig(
      name='encryption_config', help_text='The name of {resource} to use.'
  )


def OrganizationAttributeConfig():
  return concepts.ResourceParameterAttributeConfig(
      name='organization', help_text='The name of {resource} to use.'
  )


def AddDatascanResourceArg(parser, verb, positional=True):
  """Adds a resource argument for a Dataplex Datascan."""
  name = 'datascan' if positional else '--datascan'
  return concept_parsers.ConceptParser.ForResource(
      name,
      GetDatascanResourceSpec(),
      'Arguments and flags that define the Dataplex datascan you want {}'
      .format(verb),
      required=True,
  ).AddToParser(parser)


def AddProjectArg(parser, verb, positional=True):
  """Adds a resource argument for a project."""
  name = 'project' if positional else '--project'
  return concept_parsers.ConceptParser.ForResource(
      name,
      GetProjectSpec(),
      'Arguments and flags that define the project you want {}'.format(verb),
      required=True,
  ).AddToParser(parser)


def TaskAttributeConfig():
  return concepts.ResourceParameterAttributeConfig(
      name='task', help_text='The identifier of the Dataplex task resource.'
  )


def DataProductAttributeConfig():
  return concepts.ResourceParameterAttributeConfig(
      name='data_product',
      help_text='The identifier of the Dataplex data product resource.',
  )


def AddLakeResourceArg(parser, verb, positional=True):
  """Adds a resource argument for a Dataplex Lake."""
  name = 'lake' if positional else '--lake'
  return concept_parsers.ConceptParser.ForResource(
      name,
      GetLakeResourceSpec(),
      'Arguments and flags that define the Dataplex lake you want {}'.format(
          verb
      ),
      required=True,
  ).AddToParser(parser)


def AddZoneResourceArg(parser, verb, positional=True):
  """Adds a resource argument for a Dataplex Zone."""
  name = 'zone' if positional else '--zone'
  return concept_parsers.ConceptParser.ForResource(
      name,
      GetZoneResourceSpec(),
      'Arguments and flags that define the Dataplex zone you want {}'.format(
          verb
      ),
      required=True,
  ).AddToParser(parser)


def AddAssetResourceArg(parser, verb, positional=True):
  """Adds a resource argument for a Dataplex Asset."""
  name = 'asset' if positional else '--asset'
  return concept_parsers.ConceptParser.ForResource(
      name,
      GetAssetResourceSpec(),
      'Arguments and flags that define the Dataplex asset you want {}'.format(
          verb
      ),
      required=True,
  ).AddToParser(parser)


def AddContentitemResourceArg(parser, verb, positional=True):
  """Adds a resource argument for a Dataplex Content."""
  name = 'content' if positional else '--content'
  return concept_parsers.ConceptParser.ForResource(
      name,
      GetContentitemResourceSpec(),
      'The Content {}'.format(verb),
      required=True,
  ).AddToParser(parser)


def AddTaskResourceArg(parser, verb, positional=True):
  """Adds a resource argument for a Dataplex Task."""
  name = 'task' if positional else '--task'
  return concept_parsers.ConceptParser.ForResource(
      name,
      GetTaskResourceSpec(),
      'Arguments and flags that define the Dataplex task you want {}'.format(
          verb
      ),
      required=True,
  ).AddToParser(parser)


def AddDataProductResourceArg(parser, verb, positional=True):
  """Adds a resource argument for a Dataplex Data Product."""
  name = 'data_product' if positional else '--data-product'
  return concept_parsers.ConceptParser.ForResource(
      name,
      GetDataProductResourceSpec(),
      'Arguments and flags that define the Dataplex data product you want {}'
      .format(verb),
      required=True,
  ).AddToParser(parser)


def AddEnvironmentResourceArg(parser, verb, positional=True):
  """Adds a resource argument for a Dataplex Environment."""
  name = 'environment' if positional else '--environment'
  return concept_parsers.ConceptParser.ForResource(
      name,
      GetEnvironmentResourceSpec(),
      'The Environment {}'.format(verb),
      required=True,
  ).AddToParser(parser)


def AddDataTaxonomyResourceArg(parser, verb, positional=True):
  """Adds a resource argument for a Dataplex Data Taxonomy."""
  name = 'data_taxonomy' if positional else '--data_taxonomy'
  return concept_parsers.ConceptParser.ForResource(
      name,
      GetDataTaxonomyResourceSpec(),
      'The DataTaxonomy {}'.format(verb),
      required=True,
  ).AddToParser(parser)


def AddAttributeResourceArg(parser, verb, positional=True):
  """Adds a resource argument for a Dataplex Attribute."""
  name = 'data_attribute' if positional else '--data_attribute'
  return concept_parsers.ConceptParser.ForResource(
      name,
      GetDataAttributeResourceSpec(),
      'The DataAttribute {}'.format(verb),
      required=True,
  ).AddToParser(parser)


def AddDataAttributeBindingResourceArg(parser, verb, positional=True):
  """Adds a resource argument for a Dataplex DataAttributeBinding."""
  name = 'data_attribute_binding' if positional else '--data_attribute_binding'
  return concept_parsers.ConceptParser.ForResource(
      name,
      GetDataAttributeBindingResourceSpec(),
      'The DataAttributeBinding {}'.format(verb),
      required=True,
  ).AddToParser(parser)


def AddDataplexEntryGroupResourceArg(parser, verb, positional=True):
  """Adds a resource argument for a Dataplex EntryGroup."""
  name = 'entry_group' if positional else '--entry-group'
  return concept_parsers.ConceptParser.ForResource(
      name,
      GetDataplexEntryGroupResourceSpec(),
      'Arguments and flags that define the Dataplex entry group you want {}'
      .format(verb),
      required=True,
  ).AddToParser(parser)


def AddDataplexAspectTypeResourceArg(parser, verb, positional=True):
  """Adds a resource argument for a Dataplex AspectType."""
  name = 'aspect_type' if positional else '--aspect_type'
  return concept_parsers.ConceptParser.ForResource(
      name,
      GetDataplexAspectTypeResourceSpec(),
      'Arguments and flags that define the Dataplex aspect type you want {}'
      .format(verb),
      required=True,
  ).AddToParser(parser)


def AddDataplexEntryTypeResourceArg(parser, verb, positional=True):
  """Adds a resource argument for a Dataplex EntryType."""
  name = 'entry_type' if positional else '--entry_type'
  return concept_parsers.ConceptParser.ForResource(
      name,
      GetDataplexEntryTypeResourceSpec(),
      'Arguments and flags that define the Dataplex entry type you want {}'
      .format(verb),
      required=True,
  ).AddToParser(parser)


def AddEntryTypeResourceArg(parser):
  """Adds a resource argument for a Dataplex EntryType."""
  return concept_parsers.ConceptParser.ForResource(
      '--entry-type',
      GetEntryTypeResourceSpec(),
      'Arguments and flags that define the Dataplex EntryType you want to'
      ' reference.',
      required=True,
  ).AddToParser(parser)


def AddEntryResourceArg(parser):
  """Adds a resource argument for a Dataplex Entry."""
  entry_data = yaml_data.ResourceYAMLData.FromPath('dataplex.entry')
  return concept_parsers.ConceptParser.ForResource(
      'entry',
      concepts.ResourceSpec.FromYaml(entry_data.GetData(), is_positional=True),
      'Arguments and flags that define the Dataplex Entry you want to'
      ' reference.',
      required=True,
  ).AddToParser(parser)


def AddDataplexEntryLinkResourceArg(parser, verb, positional=True):
  """Adds a resource argument for a Dataplex EntryLink."""
  name = 'entry_link' if positional else '--entry-link'
  return concept_parsers.ConceptParser.ForResource(
      name,
      GetDataplexEntryLinkResourceSpec(),
      'Arguments and flags that define the Dataplex entry link you want {}'
      .format(verb),
      required=True,
  ).AddToParser(parser)


def AddGlossaryResourceArg(parser, verb, positional=True):
  """Adds a resource argument for a Dataplex Glossary."""
  name = 'glossary' if positional else '--glossary'
  return concept_parsers.ConceptParser.ForResource(
      name,
      GetGlossaryResourceSpec(),
      'Arguments and flags that define the Dataplex Glossary you want {}'
      .format(verb),
      required=True,
  ).AddToParser(parser)


def AddGlossaryCategoryResourceArg(parser, verb, positional=True):
  """Adds a resource argument for a Dataplex Glossary Category."""
  name = 'glossary_category' if positional else '--glossary_category'
  return concept_parsers.ConceptParser.ForResource(
      name,
      GetGlossaryCategoryResourceSpec(),
      'Arguments and flags that define the Dataplex Glossary Category you'
      ' want {}'.format(verb),
      required=True,
  ).AddToParser(parser)


def AddGlossaryTermResourceArg(parser, verb, positional=True):
  """Adds a resource argument for a Dataplex Glossary Term."""
  name = 'glossary_term' if positional else '--glossary_term'
  return concept_parsers.ConceptParser.ForResource(
      name,
      GetGlossaryTermResourceSpec(),
      'Arguments and flags that define the Dataplex Glossary Term you'
      ' want {}'.format(verb),
      required=True,
  ).AddToParser(parser)


def AddEncryptionConfigResourceArg(parser, verb, positional=True):
  """Adds a resource argument for a Dataplex EncryptionConfig."""
  name = 'encryption_config' if positional else '--encryption_config'
  return concept_parsers.ConceptParser.ForResource(
      name,
      GetEncryptionConfigResourceSpec(),
      'Arguments and flags that define the Dataplex EncryptionConfig you'
      ' want {}'.format(verb),
      required=True,
  ).AddToParser(parser)


def AddParentEntryResourceArg(parser):
  """Adds a resource argument for a Dataplex Entry parent."""
  entry_data = yaml_data.ResourceYAMLData.FromPath('dataplex.entry')
  return concept_parsers.ConceptParser.ForResource(
      '--parent-entry',
      concepts.ResourceSpec.FromYaml(entry_data.GetData()),
      'Arguments and flags that define the parent Entry you want to reference.',
      # Parent Entry has to belong to the same EntryGroup as the Entry,
      # therefore we disable the location and entry_group flags for the parent
      # entry resource and set fallthrougs to use the same location and
      # entry_group as for the Entry.
      command_level_fallthroughs={
          'location': ['--location'],
          'entry_group': ['--entry_group'],
      },
      flag_name_overrides={
          'location': '',
          'entry_group': '',
      },
  ).AddToParser(parser)


def AddMetadataJobResourceArg(parser, verb, positional=True):
  """Adds a resource argument for a Dataplex MetadataJob."""
  name = 'metadata_job' if positional else '--metadata_job'
  return concept_parsers.ConceptParser.ForResource(
      name,
      GetMetadataJobResourceSpec(),
      'Arguments and flags that define the Dataplex metdata job you want {}'
      .format(verb),
      required=True,
  ).AddToParser(parser)
